;        PIC16C55 130MHz FREQUENCY COUNTER WITH IF AND BFO OFFSETS.
;
;         Mick Hodges         email-mick@g4ope.demon.co.uk
;
;The circuit and board layout for this frequency counter appeared
;in SPRAT issue number 87, Summer 1996.
;
;        Prescaled version.
;        Input divided by 4 using a 74F112, dual J-K Flip Flop.
;        Choice of 4 IF offsets, 455Khz, 4.433Mhz, 9Mhz or 10.7Mhz.
;        Gate periods - No offset - automatic 4ms, 40ms and 400ms.
;                       Resolution - 1Khz, 100hz or 10hz.  
;                     - Offset selected - 4ms
;        BFO offset of 2Khz added or subtracted from display.
;        Display routine adjusted to take just under 4 ms.
;        Display updated during each gate period.
;        This version reguires 5 transistors connected to port B.
;        Code modified to convert negative counts to positive
;        when subtracting offsets.
;
; For use with a 4 Mhz crystal.
; Connect as follows :
;                             
;                           
;                             RA0       TOCK1
;                             RA1  RA2  
;                              0    0   455Khz
;                              0    1   4.433Mhz
;                              1    0   9Mhz
;                              1    1   10.7Mhz
;                             RA3       BFO offset, 0 = +  1 = -
;                             RB0       BFO offset, 0 = No 1 = Yes
;                             RB1       IF offset,  0 = +  1 = -
;                             RB2       IF offset,  0 = No 1 = Yes
;                             RB3       Digit 1
;                             RB4       Digit 2
;                             RB5       Digit 3
;                             RB6       Digit 4
;                             RB7       Digit 5
;                             RC0       segment a via 220 ohm resistor
;                              :        :             :               :
;                             RC6       segment g via 220 ohm resistor
;                             RC7       decimal point

;
          LIST P=16c55
          LIST N=66
;
include   <PIC16c5X.inc>
;
count     equ       08
H_byte    equ       09
L_byte    equ       0A
digit1    equ       0B                  ;do not move digit1 to digit5
digit2    equ       0C
digit3    equ       0D
digit4    equ       0E
digit5    equ       0F
temp      equ       10    
delay1    equ       11
delay2    equ       12
temp_h    equ       13
temp_l    equ       14
acc_h     equ       15
acc_l     equ       16
d_point   equ       17
overflow  equ       18
result    equ       19
;
; 17 bit binary to BCD conversion 
; Input  - 16 bit number in H_byte and L_byte.
;          17th bit in overflow msb.
; Output - 5 digit BCD number in digit5 through digit1. MSD = digit5.
; Returns with H_byte and L_byte unaltered.
;
B2_BCD
          movf      H_byte,w
          movwf     temp_h
          movf      L_byte,w
          movwf     temp_l
          bcf       carry_bit
          movlw     d'17'
          movwf     count
          clrf      digit3
          clrf      digit2
          clrf      digit1
loop16
          rlf       temp_l,f            ;bits 0 - 7
          rlf       temp_h,f            ;bits 8 - 15
          rlf       overflow,f          ;17th bit stored in msb of overflow
          rlf       digit1,f
          rlf       digit2,f
          rlf       digit3,f
          rrf       overflow,f          ;rotate lsb into msb position
          rrf       overflow,f
          decfsz    count,f
          goto      adjDEC
          movf      digit3,w
          andlw     b'00001111'         ;mask off the "sixth" digit.
          movwf     digit5
          movf      digit2,w
          movwf     temp
          swapf     temp,w
          andlw     b'00001111'
          movwf     digit4
          movf      digit2,w
          andlw     b'00001111'
          movwf     digit3
          movf      digit1,w
          movwf     temp
          swapf     temp,w
          andlw     b'00001111'
          movwf     digit2
          movf      digit1,w
          andlw     b'00001111'
          movwf     digit1
          retlw     0x00
adjDEC
          movlw     digit1
          movwf     fsr
          call      adjBCD
          movlw     digit2
          movwf     fsr
          call      adjBCD
          movlw     digit3
          movwf     fsr
          call      adjBCD
          goto      loop16
adjBCD
          movlw     0x03
          addwf     indf,w
          movwf     temp
          btfsc     temp,3
          movwf     indf
          movlw     0x30
          addwf     indf,w
          movwf     temp
          btfsc     temp,7
          movwf     indf
          retlw     0x00
;
; Routine to convert the BCD digit to the correct code to output on port C.
; Input  - BCD digit in w.
; Output - code in w.
;
get_code
          addwf     pcl,f
          retlw     b'00111111'         ;0
          retlw     b'00000110'         ;1
          retlw     b'01011011'         ;2
          retlw     b'01001111'         ;3
          retlw     b'01100110'         ;4
          retlw     b'01101101'         ;5
          retlw     b'01111101'         ;6
          retlw     b'00000111'         ;7
          retlw     b'01111111'         ;8
          retlw     b'01100111'         ;9
          retlw     b'00000000'         ;leading zero, (blank).
;
; Routine to read the 16 bit number stored in TMR0 and the prescaler.
; Prescaler is clocked until it rolls over and increments TMR0.
; Prescaler value calculated as 256 - no. of clocks required.
; Input  - none
; Output - 16 bit number in H_byte and L_byte.
;
get_count   
          movf      tmr0,w
          movwf     H_byte              ;high byte stored
          clrf      count
          bcf       zero_bit
clock
          bsf       ra0                 ;clock high
          bcf       ra0                 ;clock low
          incf      count,f
          movf      H_byte,w
          subwf     tmr0,w  
          btfsc     zero_bit            ;still the same if set
          goto      clock               ;so clock again
          movf      count,w             
          clrf      temp
          subwf     temp,w
          movwf     L_byte
          retlw     0x00
; Delay routine.
; Wait1 - 1ms
; When the instructions to open and ground TOCK1 are added to this delay 
; routine the delay is exact. 
; Input  - none
; Output - none
;
wait1
          movlw     d'10'
          movwf     delay1
loop10     
          movlw     d'19'
          movwf     delay2
loop100
          nop
          nop
          decfsz    delay2,f
          goto      loop100
          decfsz    delay1,f
          goto      loop10
          nop
          nop
          retlw     0x00
;
; Routines to check if the count in H_byte and L_byte is less than 16384,
; 10000 or 06554.
; Subtract 16384, 10000 or 06554 and check for carry.
; If less than 16384, 10000 or 06554 returns with w = 1, else w = 0.
;
chk_zero
          movf      L_byte,w
          btfss     zero_bit
          retlw     0x01                ;count not zero
          movf      H_byte,w
          btfss     zero_bit
          retlw     0x01                ;count not zero
          retlw     0x00                ;count = zero
chk_16384
          call      chk_zero
          addwf     pcl,f
          retlw     0x01                ;count = zero
          movf      H_byte,w
          movwf     temp_h
          movf      L_byte,w
          movwf     temp_l
          movlw     0x3f                ;high byte of 16383
          movwf     acc_h
          movlw     0xff                ;low byte of 16383
          movwf     acc_l
          goto      neg16
chk_10000
          call      chk_zero
          addwf     pcl,f
          retlw     0x01                ;count = zero
          movf      H_byte,w
          movwf     temp_h
          movf      L_byte,w
          movwf     temp_l
          movlw     0x27                ;high byte of 9999
          movwf     acc_h
          movlw     0x0f                ;low byte of 9999
          movwf     acc_l
          goto      neg16
chk_6554
          call      chk_zero
          addwf     pcl,f
          retlw     0x01                ;count = zero
          movf      H_byte,w
          movwf     temp_h
          movf      L_byte,w
          movwf     temp_l
          movlw     0x19                ;high byte of 06553
          movwf     acc_h
          movlw     0x99                ;low byte of 06553
          movwf     acc_l
neg16
          comf      temp_l,f
          incf      temp_l,f
          btfsc     zero_bit
          decf      temp_h,f
          comf      temp_h,f
add16
          movf      temp_l,w
          addwf     acc_l,f
          btfsc     carry_bit
          incf      acc_h,f
          movf      temp_h,w
          addwf     acc_h,f
          btfss     carry_bit
          retlw     0x00                
          retlw     0x01              
;
; Routine to blank the leading digit if zero.
; Only called when an offset is used and gate times limited to 4ms.
;
blank
          
          movf      digit5,f
          btfss     zero_bit
          retlw     0x00
          movlw     d'10'
          movwf     digit5
          retlw     0x00
;
; Routine to update the display.
; Each digit is displayed for 150us, all 5 digits are displayed 5 times.
; Input  - BCD in digit5 - digit1. These file registers are not altered.
; Output - display data on port pins.
;
display
          movlw     d'5'
          movwf     count
          clrf      temp
first_digit
          movlw     digit1
          movwf     fsr
next_digit
          clrf      portb               ;switch all digits off
          movf      indf,w              ;BCD in w
          call      get_code            ;code now in w
          movwf     portc               ;output digit
          movf      temp,w              ;get last digit switched on
          btfsc     zero_bit            ;no digit switched on?
          goto      sel_digit1
rotate
          bcf       carry_bit           ;set temp to point to next digit
          rlf       temp,f
          btfss     carry_bit           ;set if digit 5 lighted last
          goto      switch_on    
sel_digit1
          movlw     b'00001000'         ;digit 1
          movwf     temp
switch_on
          movf      temp,w
          subwf     d_point,w           ;zero if this digit got d.p.
          btfsc     zero_bit
          bsf       rc7                 ;set decimal point
          movf      temp,w
          movwf     portb               ;switch the digit on
          movlw     d'43'
          movwf     delay1
wait150
          decfsz    delay1,f
          goto      wait150
          incf      fsr,f               ;point to next digit
          btfss     fsr,4               ;all digits done?
          goto      next_digit          ;no 
          decfsz    count,f
          goto      first_digit         ;display all 5 again
          clrf      portb 
          retlw     0x00
;
;Routine to add or subtract 2Khz for BFO offset.
;
add_BFO
          movlw     0x00
          movwf     acc_h
          movlw     0x02
          movwf     acc_l
          btfss     ra3                 ;+ or -?
          goto      add
          goto      sub
;
; Routine to adjust the count stored in H_byte and L_byte by the IF offset
; programmed by the user on pins 2, 1 and 0 of port B.
; Pin 2, low add, high subtract.
; Pins 1 and 0, 00 = 455, 01 = 4433, 10 = 9000, 11 = 10700.
; Input  - Count in H_byte and L_byte.
; Output - Adjusted count in H_byte and L_byte.
;
add_IF
          movf      porta,w             ;read the offset
          andlw     b'00000110'
          movwf     temp
          bcf       carry_bit
          rrf       temp,f
          movf      temp,w
          addwf     pcl,f
          goto      _455                ;000
          goto      _4433               ;001
          goto      _9000               ;010
          goto      _10700              ;011
_455
          movlw     d'1'
          movwf     acc_h
          movlw     d'199'
          movwf     acc_l
          goto      add_or_sub
_4433
          movlw     d'17'
          movwf     acc_h
          movlw     d'81'
          movwf     acc_l
          goto      add_or_sub
_9000
          movlw     d'35'
          movwf     acc_h
          movlw     d'40'
          movwf     acc_l
          goto      add_or_sub
_10700
          movlw     d'41'
          movwf     acc_h
          movlw     d'204'
          movwf     acc_l
add_or_sub
          btfss     rb1
          goto      add                 
sub                                     ;get the 2's compliment
          comf      acc_l,f
          incf      acc_l,f
          btfsc     zero_bit
          decf      acc_h,f
          comf      acc_h,f
add                                     ;then add
          movf      acc_l,w
          addwf     L_byte,f
          btfsc     carry_bit
          incf      H_byte,f
          movf      acc_h,w
          addwf     H_byte,f
          btfss     rb1                 ;just subtracted offset if set
          retlw     0x00
          movlw     0xd6                ;if count now higher than 54784
          subwf     H_byte,w            ;assume it to be negative
          btfss     carry_bit
          retlw     0x00
          comf      L_byte,f            ;compliment count and add one
          comf      H_byte,f            ;to give positive count
          movlw     1
          addwf     L_byte,f
          btfss     carry_bit
          retlw     0x00
          incf      H_byte,f
          retlw     0x00
;
; It all starts here.
;
; Option - TMR0 incremented externally on high to low transition.
;          Prescaler assigned to TMR0 and set to 256.
;
set_up
          clrw
          movwf     porta               ;all low
          movwf     portb               ;all low
          movwf     portc               ;all low
          tris      portc               ;all output low     
          movlw     b'00001110'         
          tris      porta               ;RA0 output low, the rest input 
          movlw     b'00000111'         
          tris      portb               ;RB0 - 2 input the rest output low
          movlw     b'00110111'
          option
          clrf      d_point
          movlw     b'00001000'
          movwf     digit1
          movwf     digit2
          movwf     digit3
          movwf     digit4
          movwf     digit5
          clrf      delay2
testdisplay
          call      display
          decfsz    delay2,f
          goto      testdisplay
main
          clrf      tmr0                ;clear the count
gate1ms
          movlw     b'00001111'
          tris      porta               ;RA0 input ( now counting )
          call      wait1
          bcf       ra0                 ;latch low   
          movlw     b'00001110'   
          tris      porta               ;RA0 output low ( stop count )
          call      get_count
          call      chk_16384
          movwf     result
          btfss     result,0            ;<16384 if set i.e. <65.536Mhz
          goto      setoflow
          clrf      overflow
          goto      gate4ms
setoflow
          movlw     b'10000000'
          movwf     overflow
gate4ms   
          clrf      tmr0
          movlw     b'00001111'
          tris      porta               ;RA0 input ( now counting )
          call      display             ;display routine just under 4ms
          movlw     d'14'
          movwf     delay2
pad                                     ;pad to exactly 4ms
          decfsz    delay2,f
          goto      pad
          bcf       ra0                 ;latch low   
          movlw     b'00001110'   
          tris      porta               ;RA0 output low ( stop count )
          call      get_count           ;count in H_byte and L_byte
          btfss     rb2                 ;IF offset if set
          goto      no_offset
          call      add_IF           
          btfsc     rb0                 ;BFO offset if set
          call      add_BFO
          goto      setdp4ms            ;offset set so display
no_offset
          btfsc     overflow,7          ;freq >65.535Mhz?
          goto      setdp4ms            ;yes so display
          call      chk_10000           ;check if count is less than 10000
          movwf     result
          btfss     result,0            ;<10000 if set
          goto      setdp4ms
          call      chk_6554            ;check if count is less than 6554
          movwf     result
          btfss     result,0            ;<6554 if set
          goto      set17th
          goto      gate40
set17th
          movlw     b'10000000'         ;set 17th bit
          movwf     overflow
gate40
          clrf      tmr0                ;clear the count
          movlw     b'00001111'
          tris      porta               ;RA0 input ( now counting )
          movlw     d'10'
          movwf     delay2
rptt
          call      display             ;routine takes just under 4ms
          decfsz    delay2,f
          goto      rptt
          movlw     d'141'
          movwf     delay2
padd                                    ;pad to exactly 40ms
          decfsz    delay2,f            
          goto      padd
          nop
          nop
          bcf       ra0                 ;latch low   
          movlw     b'00001110'         
          tris      porta               ;RA0 output low ( stop count )
          call      get_count
          btfsc     overflow,7          ;freq >6.554Mhz?
          goto      setdp40ms           ;yes so display
          call      chk_10000           ;check if count is less than 10000
          movwf     result
          btfss     result,0            ;<10000 if set
          goto      setdp40ms
          call      chk_6554            ;check if count is less than 6554
          movwf     result
          btfss     result,0            ;<6554 if set
          goto      set17thbit
          goto      gate400
set17thbit
          movlw     b'10000000'         ;set 17th bit
          movwf     overflow
gate400
          clrf      tmr0                ;clear the count
          movlw     b'00001111'
          tris      porta               ;RA0 input ( now counting )
          movlw     d'101'
          movwf     delay2
rpt
          call      display             ;routine takes just under 40ms
          decfsz    delay2,f            ;repeat display 10 times
          goto      rpt
          movlw     d'112'
          movwf     delay2
paddd                                   ;pad to exactly 400ms
          decfsz    delay2,f            
          goto      paddd
          nop
          nop
          bcf       ra0                 ;latch low   
          movlw     b'00001110'         
          tris      porta               ;RA0 output low ( stop count )
          call      get_count
setdp400ms
          movlw     b'00100000'         ;RH d.p. digit 3 
          movwf     d_point
          goto      convert
setdp40ms
          movlw     b'10000000'         ;RH d.p. digit 5 
          movwf     d_point
          goto      convert
setdp4ms
          movlw     b'01000000'         ;RH d.p. digit 4 
          movwf     d_point
convert
          call      B2_BCD              ;digits in digit1 - digit5
          btfsc     rb2                 ;blank any leading zero 
          call      blank               ;if IF offset used
_display
          movlw     d'100'
          movwf     delay2
repeat                                  ;call display routine 100 times
          call      display             ;100 x 4ms = 400ms
          decfsz    delay2,f
          goto      repeat
          goto      main
;
          org       0x1ff
          goto      set_up
          END
