;         PIC16C74 software for Direct Digital Synthesiser board.

;         Version 1.94

;The circuit for this synthesiser appeared in SPRAT issue number 89
;Winter 1996/97. Since the published article, extensive modifications
;have been made to the code and many additional features added.

;The board was designed to replace the usual free running VFO commonly
;found in multi-band HF transceivers. Logic switching is also included
;for up to ten bands and five modes, eliminating any mechanical band
;and mode switching.
;Additionally, the DDS can be used as a general purpose signal generator
;allowing any frequency from DC to 25MHz, in 1Hz steps to be synthesised.

;         Mick Hodges         mick@g4ope.demon.co.uk


;Other devices

;AD7008                DDS IC          Frequency generator
;74HC595               Shift register  Band & Mode logic
;74ACT163              Binary counter  Clock divider
;74HC4066              Analog switch   Output selection
;93LC66                Serial EEPROM   Memories

;50MHz CMOS clock

;PIC using 3.125MHz clock (50/16)

;Features

;Direct frequency entry, (also programmes the band switches).
;Shaft encoder with steps of 500, 200, 100, 50, 20, 10, 5, 2 or 1Hz.
;Configurable for up to 10 bands, ignoring entry of frequencies out of band.
;Output configurable for each band, either 5-6MHz or 6-5MHz (5.5 - 4.5MHz).
;Band and direction information stored to eeprom.
;Five modes, USB, LSB, CW, AM and FM.
;BFO offsets, On/Off and direction configurable for each band.
;100 Memories, storing frequency and mode with store, recall and erase.
;Powers up with the last entered frequency and mode and reads configuration.
;Band sweep and memory scan features.
;Sweep or scan stopped with key entry, encoder or by grounding dedicated pin.
;Second mode - output of displayed frequency, DC-25MHz resolving to 1Hz.
;Keyboard beep that can be switched on or off.

;Function 1   - RIT                     ON, OFF or RECALL (+/- 10KHz)
;Function 2   - Mode                    USB, LSB, CW, AM or FM
;Function 3   - Tuning steps            100Hz, 10Hz, 1Hz with x2, x5
;Function 4   - Sweep band              UP/DOWN
;Function 5   - Scan memory bank        Scan one of ten memory banks
;Function 6   - BFO offsets             BFO offsets ON, OFF or SSB only
;Function 7   - Config bands            Enable or dissable any of 10 bands
;Function 8   - Config direction        Tuning direction for each band
;Function 9   - Config BFO              BFO direction for each band
;Function DOT - Beep                    Beep ON/OFF
;Function STO - Output                  5-6MHz OR DC-15MHz
;Function RCL - Scan delay              Time each memory held. 0-4.5 secs
;
;TXRX mode                              All functions
;Sig Gen mode                           Functions 3, 4, STO and RCL only
;Function RCL displays software version in Sig Gen mode

	LIST P = 16C74
          LIST N = 66

#INCLUDE <P16C74.INC>
#INCLUDE <P16CXXD.INC>


#DEFINE   LATCH1    PORTA,2
#DEFINE   LATCH2    PORTA,3
#DEFINE   ENABLE    PORTA,4
#DEFINE   SPEAKER   PORTA,4
#DEFINE   DO        PORTA,5
#DEFINE   LOAD      PORTA,1
#DEFINE   _WR       PORTC,3
#DEFINE   E         PORTC,4
#DEFINE   R_WR      PORTC,5
#DEFINE   RS        PORTC,6
#DEFINE   _CS       PORTC,7
#DEFINE   TC0       PORTD,0     
#DEFINE   TC1       PORTD,1 
#DEFINE   TC2       PORTD,2 
#DEFINE   TC3       PORTD,3
#DEFINE   DI        PORTE,0
#DEFINE   A         PORTE,0
#DEFINE   SDAT      PORTE,0
#DEFINE   CLK       PORTE,1
#DEFINE   SCLK      PORTE,1
#DEFINE   CS        PORTE,2
#DEFINE   DIN       EEPROM,7
#DEFINE   DOUT      EEPROM,6
#DEFINE   CCP1      PORTC,2
#DEFINE   CCP2      PORTC,1   

DATA_BUS            EQU       PORTD
LSD                 EQU       H'20'     ;BUFFER FOR EEPROM WRITE
MSD                 EQU       H'21'     ;          "
CHAR                EQU       H'22'     ;STORE FOR CHARACTER TO PRINT
TEMP                EQU       H'23'     ;GENERAL PURPOSE REGISTER
LASTKEY             EQU       H'24'     ;LASTKEY PRESSED
EEPROM              EQU       H'25'     ;BIT BUFFER
ADDR                EQU       H'26'     ;EEPROM ADDRESS
DATAI               EQU       H'27'     ;STORED DATA INPUT REG.
DATAO               EQU       H'28'     ;STORED DATA OUTPUT REG.
TXBUF               EQU       H'29'     ;TRANSMIT BUFFER
COUNT               EQU       H'2A'     ;BITS TRANSMITTED SO FAR
BITS                EQU       H'2B'     ;BITS TO TRANSMIT
BYTCNT              EQU       H'2C'     ;BYTE COUNTER FOR READ ROUTINE
HWORDH              EQU       H'2D'     ;STORE FOR EEPROM READ
HWORDL              EQU       H'2E'     ;         "
LWORDH              EQU       H'2F'     ;         "
LWORDL              EQU       H'30'     ;         "
KEYFLAG             EQU       H'31'     ;BIT 0 = FNC
				;BIT 1 = RCL
				;BIT 2 = STO
				;BIT 3 = ENTERING FREQUENCY
				;BIT 4 = 1 NUMBER ALREADY ENTERED
                                        ;BIT 5 = ERASE
FNCNO               EQU       H'32'     ;STORE FOR FUNCTION NUMBER
READDR              EQU       H'33'     ;CURRENT LCD READ ADDRESS
WRADDR              EQU       H'34'     ;CURRENT LCD WRITE ADDRESS
RXCHAR              EQU       H'35'     ;STORE FOR RECEIVED CHARACTER
DECPNT              EQU       H'36'     ;FLAG FOR DECIMAL POINT
LOCATION            EQU       H'37'     ;STORES 0 - 99
ACCU                EQU       H'38'     ;USED FOR X 10 ROUTINE
BAND                EQU       H'39'     ;STORE FOR BAND CODE
BAND1               EQU       H'3A'     ;STORE FOR BAND SWITCH INFO
BAND2               EQU       H'3B'     ;         "
FREQ00              EQU       H'3C'     ;STORE FOR 32 BIT NO. FOR DDS
FREQ01              EQU       H'3D'     ;TX FREQ
FREQ02              EQU       H'3E'     ;         "
FREQ03              EQU       H'3F'     ;         "
FREQ10              EQU       H'40'     ;STORE FOR 32 BIT NO. FOR DDS
FREQ11              EQU       H'41'     ;RX FREQ
FREQ12              EQU       H'42'     ;         "
FREQ13              EQU       H'43'     ;         "
_10MHz              EQU       H'44'     ;STORE FOR DISPLAYED FREQ (BCD)
_1MHz               EQU       H'45'     ;         "
_DOT                EQU       H'46'     ;         "
_100KHz             EQU       H'47'     ;         "
_10KHz              EQU       H'48'     ;         "
_1KHz               EQU       H'49'     ;         "
_100Hz              EQU       H'4A'     ;         "
_10Hz               EQU       H'4B'     ;         "
_1Hz                EQU       H'4C'     ;         "
ACC0                EQU       H'4D'     ;USED FOR MATHS ROUTINES
ACC1                EQU       H'4E'     ;FOR CALCULATING DDS
ACC2                EQU       H'4F'     ;PROGRAMMING WORD
ACC3                EQU       H'50'     ;         "
ACCA                EQU       H'51'     ;         "
ACCB                EQU       H'52'     ;         "
ACCC                EQU       H'53'     ;         "
ACCD                EQU       H'54'     ;         "
RIT                 EQU       H'55'     ;FLAG FOR RIT
STEP                EQU       H'56'     ;FLAG FOR STEP SIZE
SIGGEN              EQU       H'57'     ;FLAG FOR SIGGEN/TXRX MODE
MODE                EQU       H'58'     ;FLAG FOR MODE
DIRECN              EQU       H'59'     ;FLAG FOR SWEEP DIRECTION
SWEEP               EQU       H'5A'     ;FLAG FOR SWEEP MODE
ENTRY_COUNT         EQU       H'5B'     ;COUNTER FOR NO. OF ENTRIES MADE
REVERSE             EQU       H'5C'     ;FLAG FOR TUNING DIRECTION
CFGBND_1            EQU       H'5D'     ;BAND CONFIG INFO
CFGBND_2            EQU       H'5E'     ;         "
CFGDIR_1            EQU       H'5F'     ;BAND TUNING DIRECTION INFO
CFGDIR_2            EQU       H'60'     ;          "
CFGBFO_1            EQU       H'61'     ;BFO OFFSET DIRECTION
CFGBFO_2            EQU       H'62'     ;         "
BFOSIGN             EQU       H'63'     ;BFO DIRECTION FOR CURRENT BAND
OK                  EQU       H'64'     ;RETURNS RESULT OF A CALL
COUNT1              EQU       H'65'     ;GENERAL PURPOSE COUNTER
KEYCOUNT            EQU       H'66'     ;COUNT OF DIGITS ENTERED
SCAN                EQU       H'67'     ;FLAG FOR SCAN MODE
SCANCOUNT           EQU       H'68'     ;COUNTER IN SCAN ROUTINE
MEMHIGH             EQU       H'69'     ;MEMORY LOCATION WHEN SCANNING
MEMLOW              EQU       H'6A'     ;         "
CS_G                EQU       H'6B'     ;PERMANANT STORE FOR MY CALL
CS_4                EQU       H'6C'     ;         "
CS_O                EQU       H'6D'     ;         "
CS_P                EQU       H'6E'     ;         "
CS_E                EQU       H'6F'     ;         "
CS_                 EQU       H'70'     ;         "
SCANDELAY           EQU       H'71'     ;DELAY FOR SCAN ROUTINE
USEDCOUNT           EQU       H'72'     ;COUNTS NO. OF MEMORIES IN A BANK
UP_DWN              EQU       H'73'     ;HOLDS LOGIC STATE OF RC0
RIT_SIGN            EQU       H'74'     ;RAM FOR CURRENT RIT OFFSET (ASCII)
RIT_1KHz            EQU       H'75'     ;         "
RIT_100Hz           EQU       H'76'     ;         "
RIT_10Hz            EQU       H'77'     ;         "
RIT_1Hz             EQU       H'78'     ;         "
SECONDS             EQU       H'79'     ;NO. OF SECONDS SCAN DELAY
BEEPER              EQU       H'7A'     ;BEEP FLAG, ON OR OFF
EMPTY               EQU       H'7B'     ;FLAG FOR EMPTY LOCATION
_PCLATH             EQU       H'7C'     ;SHADOW FOR PCLATH
_W                  EQU       H'7D'     ;TEMPORARY STORE FOR W
_STATUS             EQU       H'7E'     ;TEMPORARY STORE FOR STATUS


          ORG H'0000'
	GOTO      START

          ORG H'0004'
          MOVWF     _W
          SWAPF     STATUS,W
          MOVWF     _STATUS
	MOVF      PCLATH,W
	MOVWF     _PCLATH
	BCF       PCLATH,3
	BCF       STATUS,RP0
	GOTO      INTSERV

;ROUTINE TO CONVERT THE NUMBER IN W (0 - 10) TO THE CORRESPONDING ASCII CODE
;(ALLMOST)

BCD_ASCII
	MOVWF     TEMP
	MOVLW     D'11'
	BCF       CARRY_BIT
	SUBWF     TEMP,W
	BTFSC     CARRY_BIT
	RETLW     0                   ;ILLEGAL  KEY (NOT 0 - 9 OR DOT)
	MOVF      TEMP,W
	ADDWF     PCL,F
	RETLW     "0"
	RETLW     "1"
	RETLW     "2"
	RETLW     "3"
	RETLW     "4"
	RETLW     "5"
	RETLW     "6"
	RETLW     "7"
	RETLW     "8"
	RETLW     "9"
	RETLW     "."

;INPUT BAND CODE (0 - 9), OUTPUT SERIAL DATA FOR 595

BCODE_SER
	MOVWF     TEMP                ;FIRST CHECK W IS LESS THAN 10
	MOVLW     D'10'
	SUBWF     TEMP,W
	BTFSC     CARRY_BIT
	RETLW     0                   ;IF NOT RETURN 0
	MOVF      TEMP,W
	ADDWF     PCL,F
	RETLW     1
	RETLW     2
	RETLW     4 
	RETLW     8 
	RETLW     D'16'           
	RETLW     D'32'
	RETLW     D'64'
	RETLW     D'128'
	RETLW     1
	RETLW     2

;IT ALL STARTS HERE

START 
	BSF       STATUS,RP0          ;BANK 1
	MOVLW     B'00000111'
	MOVWF     ADCON1              ;PORTS A AND E ALL DIGITAL PINS
	BCF       STATUS,RP0          ;BANK 0
          MOVLW     B'00010001'         
	MOVWF     PORTA               ;INITIAL PORTA LEVELS
	MOVLW     B'00100000'         ;SET DIRECTION
	BSF       STATUS,RP0          ;BANK 1
	MOVWF     TRISA
	BCF       STATUS,RP0          ;BANK 0
	CLRF      PORTB               ;LATCH ALL LOW
	MOVLW     B'11110000'         ;RB0-RB3 OUTPUT LOW RB4-RB7 INPUT
	BSF       STATUS,RP0          ;SELECT BANK 1
	MOVWF     TRISB                      
	BCF       OPTION_REG,NOT_RBPU     ;PORTB PULLUPS
	BCF       STATUS,RP0          ;BANK 0
	MOVLW     B'10001000'         ;INITIAL PORTC LEVELS
	MOVWF     PORTC
	MOVLW     B'00000111'         ;SET DIRECTION
	BSF       STATUS,RP0          ;BANK 1
	MOVWF     TRISC              
	BCF       STATUS,RP0          ;BANK 0
	CLRF      PORTD               ;LATCH LOW
	CLRF      PORTE               ;LATCH LOW 
	BSF       STATUS,RP0          ;BANK 1
	CLRF      TRISD               ;ALL OUTPUT LOW
	CLRF      TRISE               ;ALL OUTPUT LOW
	BCF       STATUS,RP0          ;BANK 0
	MOVLW     B'00000100'         ;EXT INT PIN
	MOVWF     CCP1CON             ;CAPTURE MODE EACH FALLING EDGE
          MOVLW     B'00000100'         ;ENCODER
          MOVWF     CCP2CON             ;CAPTURE MODE EACH FALLING EDGE
	CLRF      PIR1
	CLRF      PIR2                ;CLEAR CCP2 INT FLAG, AND OTHERS
	BSF       STATUS,RP0          ;BANK1     
	CLRF      PIE1
	CLRF      PIE2
	BSF       PIE1,CCP1IE         ;ENABLE CCP1 INTERRUPT
	BSF       PIE2,CCP2IE         ;ENABLE CCP2 INTERRUPT
	BCF       STATUS,RP0          ;BANK 0
	CLRF      T1CON
	CLRF      FNCNO               ;NO FUNCTION       
	CLRF      SWEEP               ;NO SWEEP MODE
          CLRF      SCAN                ;NO SCAN MODE
	CLRF      DIRECN              ;DIRECTION UP
	CLRF      RIT                 ;NO RIT
          CLRF      MODE                ;NO MODE
          CLRF      SIGGEN              ;START IN TXRX MODE
	CLRF      REVERSE             ;FORWARD TUNING
          MOVLW     H'FF'
	MOVWF     MEMLOW
          MOVLW     " "                 ;INITIALISE RIT VALUES
          MOVWF     RIT_SIGN
          BSF       PCLATH,3
          CALL      INIT_LCD            ;INITIALISE LCD
          BSF       PCLATH,3
          CALL      INIT_DDS            ;INITIALISE DDS
          BSF       PCLATH,3
          CALL      READ_CONFIG         ;READ BAND AND DIRECTION CONFIG
          BSF       PCLATH,3
          CALL      READ_00             ;READ CURRENT FREQ AND MODE
          BCF       PCLATH,3
          BCF       STATUS,RP0
          MOVLW     B'11001000'         ;SET GIE, PEIE AND RBIE
	MOVWF     INTCON              ;ENABLE INTERRUPTS

;WAIT FOR KEYPRESS

LOOP
	BTFSC     SWEEP,0             ;SWEEP FREQ IF SET
	GOTO      SWP
	BTFSC     SCAN,0              ;SCAN BANK IF SET
	GOTO      SCN          
	GOTO      LOOP
SWP       BCF       INTCON,GIE          ;DISABLE INTERRUPTS
	BTFSC     INTCON,GIE          ;DISABLED?
	GOTO      SWP                 ;NO SO TRY AGAIN
	BTFSS     DIRECN,0
	GOTO      S_U
S_D       CALL      DEC_FREQ            ;SWEEP DOWN
	BSF       PCLATH,3
	CALL      SND_DDS
	BCF       PCLATH,3
	BSF       INTCON,GIE          ;ENABLE INTERRUPTS
	GOTO      LOOP
S_U       CALL      INC_FREQ            ;SWEEP UP
	BSF       PCLATH,3
	CALL      SND_DDS
	BCF       PCLATH,3
	BSF       INTCON,GIE          ;ENABLE INTERRUPTS
	GOTO      LOOP

SCN       BCF       INTCON,GIE          ;DISABLE INTERRUPTS
	BTFSC     INTCON,GIE          ;DISABLED?
	GOTO      SCN                 ;NO SO TRY AGAIN
	MOVF      ADDR,W
	MOVWF     LOCATION
	CALL      READMEM
          BTFSC     EMPTY,0             ;BIT 0 SET IF EMPTY LOCATION
          GOTO      DECCOUNT
          INCF      USEDCOUNT,F         ;COUNT A USED MEMORY
          CALL      SET_BFO_DIR
          BSF       PCLATH,3
	CALL      PR_MODE
	BSF       PCLATH,3
          CALL      SET_DIR
	CALL      SND_DDS
	BCF       PCLATH,3
          MOVLW     B'00001111'         ;MAX 16 (FIRST POWER-UP)
          ANDWF     SCANDELAY,W
          BTFSC     ZERO_BIT
          GOTO      DECCOUNT
          MOVWF     SECONDS
WAITXSEC
          MOVLW     D'4'
	MOVWF     COUNT1
WAIT1SEC
	CALL      PAUSE
	DECFSZ    COUNT1,F
          GOTO      WAIT1SEC
          DECFSZ    SECONDS,F
          GOTO      WAITXSEC
DECCOUNT
	DECFSZ    SCANCOUNT,F         ;STARTS OFF AT 10
	GOTO      INCADDR
	GOTO      RESETMEM
INCADDR
	INCF      MEMLOW,F
	INCF      LOCATION,F
	INCF      LOCATION,F
	MOVF      LOCATION,W
	MOVWF     ADDR
	BSF       INTCON,GIE          ;ENABLE INTERRUPTS  
	GOTO      LOOP
RESETMEM
          MOVLW     2
          SUBWF     USEDCOUNT,W         ;CHECK MORE THAN ONE USED MEMORY
          BTFSS     CARRY_BIT
          GOTO      NOTENOUGH
	CLRF      MEMLOW
	MOVF      LASTKEY,W           ;BANK (0-9)
	CALL      MULT_X_10           ;FIRST MEMORY IN BANK (00-90)
	MOVWF     ADDR
	BCF       CARRY_BIT
	RLF       ADDR,F              ;EEPROM ADDRESS = MEMORY NO. X 2
	MOVLW     D'10'
	MOVWF     SCANCOUNT
          CLRF      USEDCOUNT
	BSF       INTCON,GIE          ;ENABLE INTERRUPTS  
	GOTO      LOOP
NOTENOUGH
          CLRF      SCAN                ;STOP SCANNING
	BSF       INTCON,GIE          ;ENABLE INTERRUPTS  
	GOTO      LOOP


;ROUTINE TO SEND THE COMMAND HELD IN W TO THE LCD

SND_CMD   
	BCF       STATUS,RP0
	MOVWF     CHAR
	CALL      BUSY_CHECK
	MOVF      CHAR,W
	MOVWF     DATA_BUS
          BCF       R_WR
	BCF       RS
	BSF       E
	BCF       E
	RETURN

;ROUTINE TO SEND THE ASCII CHARACTER HELD IN W TO THE LCD

SND_CHAR
	BCF       STATUS,RP0
	MOVWF     CHAR
	CALL      BUSY_CHECK
	MOVF      CHAR,W
	MOVWF     DATA_BUS
          BCF       R_WR
	BSF       RS
	BSF       E
	BCF       E
	RETURN

;ROUTINE TO READ THE CHARACTER AT THE CURRENT ADDRESS INTO W

REC_CHAR
	CALL      BUSY_CHECK
	BSF       STATUS,RP0          ;PAGE 1
	MOVLW     H'FF'
	MOVWF     TRISD               ;ALL INPUT
	BCF       STATUS,RP0          ;PAGE 0
	BSF       RS                  ;DATA MODE
          BSF       R_WR                ;SET TO READ
	BSF       E
	BCF       E
	MOVF      DATA_BUS,W          ;READ DATA AT CURRENT ADDRESS
          BCF       R_WR
	BSF       STATUS,RP0
	CLRF      TRISD               ;ALL OUTPUT
	BCF       STATUS,RP0          ;PAGE 0
	RETURN

;THIS ROUTINE WAITS UNTIL THE LCD IS NO LONGER BUSY BEFORE RETURNING

BUSY_CHECK
	BSF       STATUS,RP0          ;PAGE 1
	MOVLW     H'FF'
	MOVWF     TRISD               ;ALL INPUT
	BCF       STATUS,RP0          ;PAGE 0
	BCF       RS                  ;COMMAND MODE
          BSF       R_WR                ;SET TO READ BUSY FLAG
	BSF       E
	BCF       E
	MOVF      DATA_BUS,W          ;READ BUSY FLAG, DDRAM ADDRESS
	MOVWF     TEMP
	BTFSC     TEMP,7              ;HIGH = BUSY
	GOTO      BUSY_CHECK
          BCF       R_WR
	BSF       STATUS,RP0
	CLRF      TRISD               ;ALL OUTPUT
	BCF       STATUS,RP0          ;PAGE 0
	RETURN

;ROUTINE THAT RETURNS WITH THE KEY PRESSED IN W. (0 to 15, 16 = NO KEY)

GETKEY
	BTFSS     RB4
	GOTO      ROW1
	BTFSS     RB5
	GOTO      ROW2
	BTFSS     RB6
	GOTO      ROW3
	BTFSS     RB7
	GOTO      ROW4
	RETLW     D'16'
ROW1      BSF       RB0
	BTFSC     RB4
	RETLW     D'1'                ;1
	BSF       RB1
	BTFSC     RB4
	RETLW     D'2'                ;2
	BSF       RB2
	BTFSC     RB4
	RETLW     D'3'                ;3
	BSF       RB3
	BTFSC     RB4
	RETLW     D'15'               ;FNC
	RETLW     D'16'
ROW2      BSF       RB0
	BTFSC     RB5
	RETLW     D'4'                ;4
	BSF       RB1
	BTFSC     RB5
	RETLW     D'5'                ;5
	BSF       RB2
	BTFSC     RB5
	RETLW     D'6'                ;6
	BSF       RB3
	BTFSC     RB5
	RETLW     D'14'               ;RCL
	RETLW     D'16'
ROW3      BSF       RB0
	BTFSC     RB6
	RETLW     D'7'                ;7
	BSF       RB1
	BTFSC     RB6
	RETLW     D'8'                ;8
	BSF       RB2
	BTFSC     RB6
	RETLW     D'9'                ;9
	BSF       RB3
	BTFSC     RB6
	RETLW     D'13'               ;STO
	RETLW     D'16'
ROW4      BSF       RB0
	BTFSC     RB7
	RETLW     D'11'               ;CLR
	BSF       RB1
	BTFSC     RB7
	RETLW     D'0'                ;0
	BSF       RB2
	BTFSC     RB7
	RETLW     D'10'               ;DOT
	BSF       RB3
	BTFSC     RB7
	RETLW     D'12'               ;ENT
	RETLW     D'16'

;SERVICE ROUTINE FOR CCP AND KEYBOARD INTERRUPTS

INTSERV
          BTFSC     PIR1,CCP1IF         ;CCP1 INTERRUPT
          GOTO      CCP1INT            
	BTFSC     PIR2,CCP2IF         ;CCP2 INTERRUPT
	GOTO      CCP2INT

KEYBD_INT                               ;MUST BE KEYBOARD INTERRUPT
          BCF       STATUS,RP0
	BTFSC     SWEEP,0             ;SWEEP MODE?
	GOTO      STOP_SWP
          BTFSC     SCAN,0              ;SCAN MODE?
	GOTO      STOP_SCAN
	CALL      DEBOUNCE
	MOVLW     B'11110000'         ;CHECK FOR KEY DOWN
	MOVWF     TEMP  
	MOVF      PORTB,W
	SUBWF     TEMP,F
	BTFSC     ZERO_BIT
	GOTO      DONE                ;ALL KEYS UP SO RETURN
	CALL      GETKEY              ;KEY PRESSED NOW IN W (0 - 15)
	MOVWF     LASTKEY             ;SAVE IT
          BTFSC     CFGBFO_2,2
          GOTO      MISSBEEP
          BSF       PCLATH,3
          CALL      BEEP
          BCF       PCLATH,3
MISSBEEP
	MOVF      FNCNO,W             ;ENTERING FUNCTION CHOICE?
	BTFSS     ZERO_BIT
	GOTO      GET_FNC_CHCE
	BTFSC     KEYFLAG,3           ;ENTERING FREQUENCY MODE?
          GOTO      CHKENT
CHKFNCF   BTFSS     KEYFLAG,0           ;CHECK FOR FNC MODE
	GOTO      CHKRCLF
	MOVF      LASTKEY,W           ;FNC NUMBER MUST BE IN LASTKEY
	MOVWF     FNCNO
	CALL      DISPFNC             ;DISPLAY THE CHOSEN FUNCTION SCREEN
	BCF       PCLATH,3
	GOTO      DONE
CHKRCLF   BTFSS     KEYFLAG,1           ;CHECK FOR RECALL MODE
	GOTO      CHKSTOF
	CALL      RCLMEM
	BTFSC     KEYFLAG,4           ;1 NUMBER ENTERED IF SET
	GOTO      DONE
          BSF       PCLATH,3
          CALL      DISP_MODE
	BSF       PCLATH,3
          CALL      SET_DIR
	CALL      SND_DDS
	BCF       PCLATH,3
	GOTO      DONE
CHKSTOF   BTFSS     KEYFLAG,2           ;CHECK FOR STORE MODE
          GOTO      CHKCLRF
	CALL      STOREDISPLAY
	GOTO      DONE
CHKCLRF   BTFSS     KEYFLAG,5           ;CHECK FOR ERASE MODE
          GOTO      CHKFNC
          CALL      ERASEMEM
          GOTO      DONE
CHKFNC    MOVLW     D'15'               ;FNC KEY?
	SUBWF     LASTKEY,W
	BTFSC     ZERO_BIT
	GOTO      FNC               
CHKRCL    MOVLW     D'14'               ;RCL KEY?
	SUBWF     LASTKEY,W
	BTFSC     ZERO_BIT
	GOTO      RCL
CHKSTO    MOVLW     D'13'               ;STO KEY?
	SUBWF     LASTKEY,W
	BTFSC     ZERO_BIT
	GOTO      STO
CHKCLR    MOVLW     D'11'               ;CLR KEY?
	SUBWF     LASTKEY,W
	BTFSC     ZERO_BIT
	GOTO      CLR
CHKENT    MOVLW     D'12'               ;ENTER KEY?
	SUBWF     LASTKEY,W
	BTFSC     ZERO_BIT
	GOTO      ENT
	MOVF      LASTKEY,W
	CALL      BCD_ASCII           ;MUST BE 0 TO 9 OR DOT
	MOVWF     TEMP                ;RETURNS 0 IN W IF NOT
	MOVF      TEMP,W              ;TEST FOR ZERO
	BTFSC     ZERO_BIT            
          GOTO      CLRALL              ;ABORT IF SO
	BTFSS     KEYFLAG,3           ;FIRST DIGIT?
	CALL      DISPFREQ            ;YES SO DISPLAY "ENTER FREQUENCY"
	CALL      SND_CHAR            ;PRINT THE NUMBER ENTERED
	INCF      ENTRY_COUNT,F       ;COUNT NUMBERS AS THEY ARE ENTERED
	MOVLW     D'10'
	SUBWF     ENTRY_COUNT,W       ;MAXIMUM 9 NUMBERS ALLOWED
	BTFSC     ZERO_BIT
	GOTO      ENT                 ;TREAT 10TH NUMBER AS ENTER KEY
	GOTO      DONE
FNC       CALL      FUNCTION
	GOTO      DONE
RCL       BTFSS     SIGGEN,0
	CALL      RECALL
	GOTO      DONE
STO       BTFSS     SIGGEN,0
	CALL      STORE
	GOTO      DONE
CLR       BTFSS     SIGGEN,0
          CALL      CLEAR
	GOTO      DONE
ENT       CALL      ENTER
	GOTO      DONE
CLRALL
          CALL      CLRPAGE2
          GOTO      DONE

STOP_SWP
          BTFSC     CFGBFO_2,2
          GOTO      MISBEEP
          BSF       PCLATH,3
          CALL      BEEP
          BCF       PCLATH,3
MISBEEP
	BCF       SWEEP,0
	GOTO      DONE
STOP_SCAN
          BTFSC     CFGBFO_2,2
          GOTO      MSBEEP
          BSF       PCLATH,3
          CALL      BEEP
          BCF       PCLATH,3
MSBEEP
	BCF       SCAN,0
DONE
	BCF       STATUS,RP0
	CLRF      PORTB
	MOVF      PORTB,F
	BCF       INTCON,RBIF         ;CLEAR FLAG
	MOVF      _PCLATH,W           ;RESET PCLATH
	MOVWF     PCLATH
          SWAPF     _STATUS,W
          MOVWF     STATUS
          SWAPF     _W,F
          SWAPF     _W,W
	RETFIE

CCP1INT
	BCF       STATUS,RP0
	BCF       SWEEP,0
	BCF       SCAN,0
	MOVF      KEYFLAG,W
	BTFSS     ZERO_BIT
          CALL      CLRPAGE2
	MOVF      FNCNO,W
	BTFSS     ZERO_BIT
          CALL      CLRPAGE2
	BCF       PIR1,CCP1IF         ;CLEAR INT FLAG 
	BCF       STATUS,RP0
	MOVF      _PCLATH,W           
	MOVWF     PCLATH
          SWAPF     _STATUS,W
          MOVWF     STATUS
          SWAPF     _W,F
          SWAPF     _W,W
	RETFIE
CCP2INT
	BCF       STATUS,RP0
	BTFSC     SWEEP,0
	GOTO      STOP_SW
	BTFSC     SCAN,0
	GOTO      STOP_SC
          MOVF      PORTC,W             ;RC0 SAVED IN UP_DWN,0
          MOVWF     UP_DWN
	MOVF      KEYFLAG,W
	BTFSS     ZERO_BIT
          CALL      CLRPAGE2
	MOVF      FNCNO,W             ;CHECK NOT IN MIDDLE OF A FUNCTION
	BTFSS     ZERO_BIT            ;OR A RECALL OR STORE
          CALL      CLRPAGE2
	MOVLW     B'10001110'         ;LCD ADDR 0E
	CALL      SND_CMD
	MOVLW     " "                 ;BLANK OFF ANY MEMORY LOCATION
	CALL      SND_CHAR
	MOVLW     " "
	CALL      SND_CHAR
          BTFSS     UP_DWN,0
	GOTO      TUNE_DWN
	CALL      INC_FREQ
	GOTO      RST_INT
TUNE_DWN  CALL      DEC_FREQ
RST_INT   BSF       PCLATH,3
	CALL      SND_DDS
	BCF       PCLATH,3
          CLRF      COUNT1
          CLRF      COUNT
CHK_RC1_H                               ;WAIT UNTIL RC1 GOES HIGH
          BTFSC     RC1                 ;FOR A MAXIMUM OF 400mS
          GOTO      CHK_RC0
          DECFSZ    COUNT,F
          GOTO      CHK_RC1_H
          DECFSZ    COUNT1,F
          GOTO      CHK_RC1_H
          GOTO      OUT
CHK_RC0
          MOVLW     D'150'
          MOVWF     COUNT1
          CLRF      COUNT
          BTFSC     UP_DWN,0
          GOTO      CHK_RC0_H
CHK_RC0_L                               ;WAIT UNTIL RC0 GOES LOW
          BTFSS     RC0                 ;FOR A MAXIMUM OF 250mS
          GOTO      OUT
          DECFSZ    COUNT,F
          GOTO      CHK_RC0_L
          DECFSZ    COUNT1,F
          GOTO      CHK_RC0_L
          GOTO      OUT
CHK_RC0_H                               ;WAIT UNTIL RC0 GOES HIGH
          BTFSC     RC0                 ;FOR A MAXIMUM OF 250mS
          GOTO      OUT
          DECFSZ    COUNT,F
          GOTO      CHK_RC0_H
          DECFSZ    COUNT1,F
          GOTO      CHK_RC0_H
OUT
	BCF       STATUS,RP0
          BCF       PIR2,CCP2IF         ;CLEAR INT FLAG
	MOVF      _PCLATH,W           
	MOVWF     PCLATH
          SWAPF     _STATUS,W
          MOVWF     STATUS
          SWAPF     _W,F
          SWAPF     _W,W
	RETFIE
STOP_SW 
	BCF       SWEEP,0
	GOTO      OUT
STOP_SC   BCF       SCAN,0
	GOTO      OUT

INC_FREQ
	BTFSC     RIT,0
	GOTO      INC_RIT
          BTFSC     STEP,3
          GOTO      TIMES2
          BTFSC     STEP,4
          GOTO      TIMES5
TIMES1    MOVLW     1
          MOVWF     COUNT
          GOTO      NOMULT
TIMES2    MOVLW     2
          MOVWF     COUNT
          GOTO      NOMULT
TIMES5    MOVLW     5
          MOVWF     COUNT
NOMULT	BTFSC     STEP,0              ;1HZ STEPS
	GOTO      STEP_1              
	BTFSC     STEP,1              ;10HZ STEPS
	GOTO      STEP_2
	MOVLW     6                   ;100HZ STEPS
	MOVWF     ADDR
	GOTO      GET_NUM
STEP_2    MOVLW     7
	MOVWF     ADDR
	GOTO      GET_NUM
STEP_1    MOVLW     8                   ;LCD ADDR 08
	MOVWF     ADDR
GET_NUM 
	BSF       ADDR,7  
	MOVF      ADDR,W
	CALL      SND_CMD
	CALL      REC_CHAR
	MOVWF     ACCU
	MOVLW     "."                 ;FINISHED IF DECIMAL POINT
	SUBWF     ACCU,W
	BTFSC     ZERO_BIT
	GOTO      SAVE
	MOVLW     "9"
	SUBWF     ACCU,W
	BTFSC     ZERO_BIT
	GOTO      NUM_9
	MOVF      ADDR,W
	CALL      SND_CMD
	INCF      ACCU,W              ;LESS THAN 9 SO INCREMENT  
	CALL      SND_CHAR
SAVE      DECFSZ    COUNT,F
          GOTO      NOMULT
          BTFSC     SWEEP,0
	CALL      DEBOUNCE
	RETURN

NUM_9
	MOVF      ADDR,W
	CALL      SND_CMD
	MOVLW     "0"                 ;REPLACE 9 WITH 0
	CALL      SND_CHAR
	BCF       ADDR,7
	DECF      ADDR,F              ;PREVIOUS NUMBER
	GOTO      GET_NUM

DEC_FREQ
	BTFSC     RIT,0
	GOTO      DEC_RIT
          BTFSC     STEP,3
          GOTO      TIMES_2
          BTFSC     STEP,4
          GOTO      TIMES_5
TIMES_    MOVLW     1
          MOVWF     COUNT
          GOTO      NO_MULT
TIMES_2   MOVLW     2
          MOVWF     COUNT
          GOTO      NO_MULT
TIMES_5   MOVLW     5
          MOVWF     COUNT
NO_MULT   BTFSC     STEP,0              ;1HZ STEPS
	GOTO      STEP1
	BTFSC     STEP,1              ;10HZ STEPS
	GOTO      STEP2
	MOVLW     6                   ;100HZ STEPS
	MOVWF     ADDR
	GOTO      GET_NUMB
STEP2     MOVLW     7
	MOVWF     ADDR
	GOTO      GET_NUMB
STEP1     MOVLW     8                   ;LCD ADDR 08
	MOVWF     ADDR
GET_NUMB 
	BSF       ADDR,7  
	MOVF      ADDR,W
	CALL      SND_CMD
	CALL      REC_CHAR
	MOVWF     ACCU
	MOVLW     "."                 ;FINISHED IF DECIMAL POINT
	SUBWF     ACCU,W
	BTFSC     ZERO_BIT
	GOTO      SAV
	MOVLW     "0"
	SUBWF     ACCU,W
	BTFSC     ZERO_BIT
	GOTO      NUM_0
	MOVF      ADDR,W
	CALL      SND_CMD
	DECF      ACCU,W              ;MORE THAN 0 SO DECREMENT  
	CALL      SND_CHAR
SAV       DECFSZ    COUNT,F
          GOTO      NO_MULT
          BTFSC     SWEEP,0
	CALL      DEBOUNCE
	RETURN

NUM_0
	MOVF      ADDR,W
	CALL      SND_CMD
	MOVLW     "9"                 ;REPLACE 0 WITH 9
	CALL      SND_CHAR
	BCF       ADDR,7
	DECF      ADDR,F              ;PREVIOUS NUMBER
	GOTO      GET_NUMB


INC_RIT
	MOVLW     B'11000100'         ;LCD ADDR 44
	CALL      SND_CMD
	CALL      REC_CHAR
	MOVWF     ACCU
	MOVLW     " "
	SUBWF     ACCU,W
	BTFSC     ZERO_BIT
	CALL      SET_PLUS
	MOVLW     "-"                 ;CHECK RX BELOW TX
	SUBWF     ACCU,W
	BTFSC     ZERO_BIT
	GOTO      SUB_RIT             ;IF SO DECREMENT RIT
	GOTO      ADD_RIT             ;ELSE INCREMENT RIT
DEC_RIT
	MOVLW     B'11000100'         ;LCD ADDR 44
	CALL      SND_CMD
	CALL      REC_CHAR
	MOVWF     ACCU
	MOVLW     " "
	SUBWF     ACCU,W
	BTFSC     ZERO_BIT
	CALL      SET_MINUS
	MOVLW     "+"                 ;CHECK RX ABOVE TX
	SUBWF     ACCU,W
	BTFSC     ZERO_BIT
	GOTO      SUB_RIT             ;IF SO DECREMENT RIT          
	GOTO      ADD_RIT             ;ELSE INCREMENT RIT  

ADD_RIT
	BTFSC     STEP,0              ;1HZ STEPS
	GOTO      STP_1
	BTFSC     STEP,1              ;10HZ STEPS
	GOTO      STP_2
	MOVLW     H'46'               ;100HZ STEPS
	MOVWF     ADDR
	GOTO      GET_NM
STP_2     MOVLW     H'47'
	MOVWF     ADDR
	GOTO      GET_NM
STP_1     MOVLW     H'48'               ;LCD ADDR 48
	MOVWF     ADDR
GET_NM 
	BSF       ADDR,7  
	MOVF      ADDR,W
	CALL      SND_CMD
	CALL      REC_CHAR
	MOVWF     ACCU
	BTFSS     ACCU,4              ;NOT A NUMBER IF CLEAR
	GOTO      C_Z                 ;SO MUST BE "+","-" OR " "
	MOVLW     "9"
	SUBWF     ACCU,W
	BTFSC     ZERO_BIT
	GOTO      NUM_09
	MOVF      ADDR,W
	CALL      SND_CMD
	INCF      ACCU,W              ;LESS THAN 9 SO INCREMENT  
	CALL      SND_CHAR
C_Z       CALL      ALL_ZEROS
          CALL      SAVE_RIT
	RETURN
NUM_09
	MOVF      ADDR,W
	CALL      SND_CMD
	MOVLW     "0"                 ;REPLACE 9 WITH 0
	CALL      SND_CHAR
	BCF       ADDR,7
	DECF      ADDR,F              ;PREVIOUS NUMBER
	GOTO      GET_NM

SUB_RIT
	BTFSC     STEP,0              ;1HZ STEPS
	GOTO      S1
	BTFSC     STEP,1              ;10HZ STEPS
	GOTO      S2
	MOVLW     H'46'               ;100HZ STEPS
	MOVWF     ADDR
	GOTO      GET_NB
S2        MOVLW     H'47'
	MOVWF     ADDR
	GOTO      GET_NB
S1        MOVLW     H'48'               ;LCD ADDR 08
	MOVWF     ADDR
GET_NB 
	BSF       ADDR,7  
	MOVF      ADDR,W
	CALL      SND_CMD
	CALL      REC_CHAR
	MOVWF     ACCU
	BTFSS     ACCU,4              ;NOT A NUMBER IF CLEAR
	GOTO      CH_Z                ;SO MUST BE "+","-" OR " "
	MOVLW     "0"
	SUBWF     ACCU,W
	BTFSC     ZERO_BIT
	GOTO      NUM_00
	MOVF      ADDR,W
	CALL      SND_CMD
	DECF      ACCU,W              ;MORE THAN 0 SO DECREMENT  
	CALL      SND_CHAR
CH_Z      CALL      ALL_ZEROS
          CALL      SAVE_RIT
	RETURN
NUM_00
	MOVF      ADDR,W
	CALL      SND_CMD
	MOVLW     "9"                 ;REPLACE 0 WITH 9
	CALL      SND_CHAR
	BCF       ADDR,7
	DECF      ADDR,F              ;PREVIOUS NUMBER
	GOTO      GET_NB

SET_PLUS
	MOVLW     B'11000100'         ;LCD ADDR 44
	CALL      SND_CMD
	MOVLW     "+"
	CALL      SND_CHAR
	BCF       RIT,7
	RETURN

SET_MINUS
	MOVLW     B'11000100'         ;LCD ADDR 44
	CALL      SND_CMD
	MOVLW     "-"
	CALL      SND_CHAR
	BSF       RIT,7
	RETURN

ALL_ZEROS
	MOVLW     B'11000101'         ;LCD ADDR 45
	CALL      SND_CMD
	CALL      REC_CHAR            ;GET FIRST NUMBER
	ANDLW     B'00001111'
	BTFSS     ZERO_BIT
	RETURN                        ;RETURN IF NOT ZERO
	CALL      REC_CHAR            ;GET SECOND
	ANDLW     B'00001111'
	BTFSS     ZERO_BIT
	RETURN
	CALL      REC_CHAR            ;GET THIRD
	ANDLW     B'00001111'
	BTFSS     ZERO_BIT
	RETURN
	CALL      REC_CHAR            ;GET LAST
	ANDLW     B'00001111'
	BTFSS     ZERO_BIT
	RETURN
	MOVLW     B'11000100'         ;LCD ADDR 44
	CALL      SND_CMD
	MOVLW     " "                 ;4 ZEROS SO DELETE SIGN
	CALL      SND_CHAR
	RETURN

SAVE_RIT
	MOVLW     B'11000100'         ;LCD ADDR 44
	CALL      SND_CMD
          MOVLW     5
          MOVWF     COUNT
          MOVLW     RIT_SIGN
          MOVWF     FSR
NEXTRIT
	CALL      REC_CHAR
          MOVWF     INDF
          INCF      FSR,F
          DECFSZ    COUNT,F
          GOTO      NEXTRIT
          RETURN

SAVE100
	BTFSS     SIGGEN,0
	GOTO      ST_200
	MOVLW     D'202'
	GOTO      STO_IT
ST_200    MOVLW     D'200'
STO_IT    MOVWF     ADDR
	CALL      STOREDISP           ;SAVE IN MEMORY 100 OR 101
	RETURN


       

;DELAY ROUTINE OF ABOUT 100mS

PAUSE
	MOVLW     D'5'
	MOVWF     COUNT
LOOP2     CALL      DEBOUNCE
	DECFSZ    COUNT,F
	GOTO      LOOP2
	RETURN

;DELAY ROUTINE OF ABOUT 25mS

DEBOUNCE                             
          MOVLW     D'25'               ;APPROX. NO. OF mS
DEL       MOVWF     MSD
	CLRF      LSD
LOOP3
	DECFSZ    LSD,F
	GOTO      LOOP3
	DECFSZ    MSD,F
	GOTO      LOOP3
	RETURN

;ROUTINE TO SERVICE THE FUNCTION CHOICE ENTERED

GET_FNC_CHCE
	BCF       STATUS,RP0
	MOVF      FNCNO,W
	BTFSC     ZERO_BIT            ;FUNCTION 0?
          GOTO      CLRALL
	MOVLW     1
	SUBWF     FNCNO,W
	BTFSC     ZERO_BIT            ;FUNCTION 1?
	GOTO      FNC1
	MOVLW     2
	SUBWF     FNCNO,W
	BTFSC     ZERO_BIT            ;FUNCTION 2?
	GOTO      FNC2
	MOVLW     3
	SUBWF     FNCNO,W
	BTFSC     ZERO_BIT            ;FUNCTION 3?
	GOTO      FNC3
	MOVLW     4
	SUBWF     FNCNO,W
	BTFSC     ZERO_BIT            ;FUNCTION 4?
	GOTO      FNC4
	MOVLW     5
	SUBWF     FNCNO,W
	BTFSC     ZERO_BIT            ;FUNCTION 5?
	GOTO      FNC5
	MOVLW     6
	SUBWF     FNCNO,W
	BTFSC     ZERO_BIT            ;FUNCTION 6?
	GOTO      FNC6
	MOVLW     7
	SUBWF     FNCNO,W
	BTFSC     ZERO_BIT            ;FUNCTION 7?
	GOTO      FNC7
	MOVLW     8
	SUBWF     FNCNO,W
	BTFSC     ZERO_BIT            ;FUNCTION 8?
	GOTO      FNC8
	MOVLW     9
	SUBWF     FNCNO,W
	BTFSC     ZERO_BIT            ;FUNCTION 9?
	GOTO      FNC9
          MOVLW     D'10'
	SUBWF     FNCNO,W
          BTFSC     ZERO_BIT            ;FUNCTION DOT?
          GOTO      FNCDOT
          MOVLW     D'13'
	SUBWF     FNCNO,W
          BTFSC     ZERO_BIT            ;FUNCTION STO?
          GOTO      FNCSTO
          MOVLW     D'14'
	SUBWF     FNCNO,W
          BTFSC     ZERO_BIT            ;FUNCTION RCL?
          GOTO      FNCRCL
          GOTO      CLRALL          

FNC1      BSF       PCLATH,3
          CALL      FUNC1
	BSF       PCLATH,3
	CALL      SND_DDS
	BCF       PCLATH,3
          GOTO      CLRALL
FNC2      BSF       PCLATH,3
          CALL      FUNC2
          BCF       PCLATH,3
          GOTO      CLRALL
FNC3      BSF       PCLATH,3
          CALL      FUNC3
          BCF       PCLATH,3
          GOTO      CLRALL
FNC4      BSF       PCLATH,3
          CALL      FUNC4
          BCF       PCLATH,3
          GOTO      CLRALL
FNC5      BSF       PCLATH,3
          CALL      FUNC5
          BCF       PCLATH,3
          GOTO      CLRALL
FNC6      BSF       PCLATH,3
          CALL      FUNC6
          BCF       PCLATH,3
          GOTO      CLRALL
FNC7      BSF       PCLATH,3
          CALL      FUNC7
          BCF       PCLATH,3
	GOTO      DONE
FNC8      BSF       PCLATH,3
          CALL      FUNC8
          BCF       PCLATH,3
	GOTO      DONE
FNC9      BSF       PCLATH,3
          CALL      FUNC9
          BCF       PCLATH,3
          GOTO      DONE
FNCDOT    BSF       PCLATH,3
          CALL      FUNCDOT
          BCF       PCLATH,3
          GOTO      CLRALL
FNCSTO    BSF       PCLATH,3
          CALL      FUNCSTO
          BCF       PCLATH,3
          GOTO      CLRALL
FNCRCL    BSF       PCLATH,3
          CALL      FUNCRCL
          BCF       PCLATH,3
          GOTO      CLRALL

;ROUTINE TO SERVICE AN ENTER KEY HIT

ENTER
	BCF       STATUS,RP0
	BTFSC     KEYFLAG,3           ;LONE ENTER KEY HIT?
	GOTO      READ_LCD            ;NO
	CALL      SAVE100             ;YES
          GOTO      CLRALL
READ_LCD
	CLRF      KEYFLAG
	CLRF      FNCNO
	CALL      CHK_DECPNT          ;CHECK ONLY 1 DP ENTERED
	MOVWF     OK
	BTFSS     OK,0
          GOTO      CLRALL
	MOVLW     9
	MOVWF     COUNT
	MOVLW     B'10000000'         ;LCD DDDR 0 
	MOVWF     WRADDR              
	MOVLW     B'11010001'         ;LCD ADDR 51
	CALL      SND_CMD   
	CALL      REC_CHAR            ;LOOK FOR DECIMAL POINT
	MOVWF     RXCHAR              ;AT POSITION 2
	MOVLW     "."
	SUBWF     RXCHAR,W
	BTFSC     ZERO_BIT
	GOTO      LOWBAND
	CALL      REC_CHAR            ;LOOK FOR DECIMAL POINT
	MOVWF     RXCHAR              ;AT POSITION 3
	MOVLW     "."
	SUBWF     RXCHAR,W
	BTFSC     ZERO_BIT
	GOTO      HIGHBAND
          GOTO      CLRALL             ;DECIMAL POINT IN WRONG PLACE
LOWBAND   
	BTFSC     SIGGEN,0            ;CHECK OPERATING MODE
	GOTO      ANYLOW
	MOVLW     B'11010000'         ;LCD ADDR 50
	CALL      SND_CMD
	CALL      ZERO_SEC            ;BAND CODE NOW IN W
	MOVWF     BAND
	MOVLW     D'10'               ;CHECK AMATEUR BAND ENTERED
	SUBWF     BAND,W
	BTFSC     ZERO_BIT
          GOTO      CLRALL
	BSF       PCLATH,3
	CALL      CHK_BAND            ;CHECK BAND ALLOWED
	BCF       PCLATH,3
	BTFSS     OK,0
          GOTO      CLRALL
          CALL      SET_BFO_DIR         ;GET BFO DIRECTION FOR THAT BAND
	BSF       PCLATH,3
	CALL      SET_DIR             ;SET DIRECTION FOR THAT BAND
	BCF       PCLATH,3
ANYLOW    DECF      COUNT,F
	MOVF      WRADDR,W
	CALL      SND_CMD   
	MOVLW     " "                 ;SPACE
	CALL      SND_CHAR
	INCF      WRADDR,F
	GOTO      MISBNDCHK
HIGHBAND  
	BTFSS     SIGGEN,0
	GOTO      AM_BAND
	CALL      CHK_MAX             ;MAX FREQ 24.999999MHz
	MOVWF     TEMP
	BTFSS     TEMP,0
          GOTO      CLRALL
	GOTO      MISBNDCHK 
AM_BAND   MOVLW     B'11010000'         ;LCD ADDR 50
	CALL      SND_CMD
	CALL      CONV_BAND           ;BAND CODE NOW IN W
	MOVWF     BAND
	MOVLW     D'10'               ;CHECK AMATEUR BAND ENTERED
	SUBWF     BAND,W
	BTFSC     ZERO_BIT
          GOTO      CLRALL
	BSF       PCLATH,3
	CALL      CHK_BAND            ;CHECK BAND ALLOWED
	BCF       PCLATH,3
	BTFSS     OK,0
          GOTO      CLRALL
          CALL      SET_BFO_DIR         ;GET BFO DIRECTION FOR THAT BAND
	BSF       PCLATH,3
	CALL      SET_DIR             ;SET DIRECTION FOR THAT BAND
	BCF       PCLATH,3
MISBNDCHK
	MOVLW     B'11010000'         ;LCD ADDR 50
	MOVWF     READDR    
NEXTCHAR
	MOVF      READDR,W
	CALL      SND_CMD            
	CALL      REC_CHAR            ;READ CHARACTER INTO W
	MOVWF     RXCHAR              
	MOVLW     " "                 ;CHECK FOR ASCII SPACE
	SUBWF     RXCHAR,W
	BTFSC     ZERO_BIT
	BSF       RXCHAR,4            ;CHANGE SPACE TO A ZERO
WRITEIT   MOVF      WRADDR,W
	CALL      SND_CMD   
	MOVF      RXCHAR,W
	CALL      SND_CHAR            ;SEND CHARACTER IN W
	INCF      READDR,F
	INCF      WRADDR,F
	DECFSZ    COUNT,F
	GOTO      NEXTCHAR
	MOVLW     B'10001110'         ;LCD ADDR 0E
	CALL      SND_CMD
	MOVLW     " "                 ;BLANK OFF ANY MEMORY LOCATION
	CALL      SND_CHAR
	MOVLW     " "
	CALL      SND_CHAR
	CALL      SAVE100             ;SAVE IN LOCATION 100 OR 101
          BSF       PCLATH,3
	CALL      DESEL_RIT           ;DESELECT RIT
	BSF       PCLATH,3
	CALL      SND_BAND            ;PROG 595'S
	BSF       PCLATH,3
	CALL      SND_DDS             ;SEND FREQ TO DDS
	BCF       PCLATH,3
          CALL      CLRPAGE2              
	RETURN
CHK_MAX
	MOVLW     B'11010000'         ;LCD ADDR 50
	CALL      SND_CMD             ;CHECK FOR GREATER THAN 2
	CALL      REC_CHAR            ;10MHz DIGIT
	ANDLW     B'00001111'
	MOVWF     TEMP
	MOVLW     3         
	SUBWF     TEMP,W
	BTFSC     CARRY_BIT
	RETLW     0                   ;30MHz - TOO HIGH
	MOVLW     2          
	SUBWF     TEMP,W
	BTFSS     ZERO_BIT
	RETLW     1                   ;10MHz - OK
	CALL      REC_CHAR            ;20MHz SO GET 1MHz DIGIT
	ANDLW     B'00001111'
	MOVWF     TEMP
	MOVLW     5
	SUBWF     TEMP,W
	BTFSC     CARRY_BIT
	RETLW     0                   ;25MHz - TOO HIGH
	RETLW     1                   ;LESS THAN 25MHz - OK

;ROUTINE TO CHECK ONLY 1 DECIMAL POINT HAS BEEN ENTERED

CHK_DECPNT
	MOVLW     1
	MOVWF     DECPNT              ;CHECK ONLY ONE DECIMAL POINT
	MOVLW     B'11010000'         ;LCD ADDR 50
	CALL      SND_CMD
	MOVLW     9
	MOVWF     COUNT
NEXT_ENT
	CALL      REC_CHAR
	MOVWF     RXCHAR
	MOVLW     "."
	SUBWF     RXCHAR,W
	BTFSC     ZERO_BIT            ;SET IF DECIMAL POINT
	DECF      DECPNT,F            ;FLAG DECIMAL POINT
	BTFSC     DECPNT,7            ;SET IF MORE THAN 1
	RETLW     0                   ;TOO MANY DECIMAL POINTS
	DECFSZ    COUNT,F
	GOTO      NEXT_ENT
	BTFSC     DECPNT,0            ;STILL SET IF NO DECIMAL POINTS
	RETLW     0                   ;NO DECIMAL POINT
	RETLW     1                   ;1 DECIMAL POINT - OK


;ROUTINE TO SERVICE AND FLAG A FUNCTION KEY HIT

FUNCTION
	BCF       STATUS,RP0
	CLRF      KEYFLAG
	BSF       KEYFLAG,0
	CALL      SETPAGE2
	MOVLW     "F"
	CALL      SND_CHAR
	MOVLW     "u"
	CALL      SND_CHAR
	MOVLW     "n"
	CALL      SND_CHAR
	MOVLW     "c"
	CALL      SND_CHAR
	MOVLW     "t"
	CALL      SND_CHAR
	MOVLW     "i"
	CALL      SND_CHAR
	MOVLW     "o"
	CALL      SND_CHAR
	MOVLW     "n"
	CALL      SND_CHAR
	MOVLW     " "
	CALL      SND_CHAR
	RETURN

DISPFREQ
          CALL      CLRPAGE2              ;WIPE PAGE 2
	CALL      SETPAGE2
	MOVLW     "E"
	CALL      SND_CHAR
	MOVLW     "n"
	CALL      SND_CHAR 
	MOVLW     "t"
	CALL      SND_CHAR
	MOVLW     "e"
	CALL      SND_CHAR 
	MOVLW     "r"
	CALL      SND_CHAR
	MOVLW     " "
	CALL      SND_CHAR 
	MOVLW     "f"
	CALL      SND_CHAR
	MOVLW     "r"
	CALL      SND_CHAR 
	MOVLW     "e"
	CALL      SND_CHAR
	MOVLW     "q"
	CALL      SND_CHAR 
	MOVLW     "u"
	CALL      SND_CHAR
	MOVLW     "e"
	CALL      SND_CHAR 
	MOVLW     "n"
	CALL      SND_CHAR
	MOVLW     "c"
	CALL      SND_CHAR 
	MOVLW     "y"
	CALL      SND_CHAR
	MOVLW     B'11010000'         ;LCD ADDR H50
	CALL      SND_CMD   
	BSF       KEYFLAG,3           ;ENTERING FREQUENCY MODE
	MOVF      LASTKEY,W
	CALL      BCD_ASCII
	RETURN


;ROUTINE TO SERVICE AND FLAG A RECALL KEY HIT

RECALL
	BCF       STATUS,RP0
	CLRF      KEYFLAG
	BSF       KEYFLAG,1           ;FLAG RECALL KEY PRESSED        
	CALL      SETPAGE2
	MOVLW     "R"
	CALL      SND_CHAR
	MOVLW     "e"
	CALL      SND_CHAR
	MOVLW     "c"
	CALL      SND_CHAR
	MOVLW     "a"
	CALL      SND_CHAR
	MOVLW     "l"
	CALL      SND_CHAR
	MOVLW     "l"
	CALL      SND_CHAR
	MOVLW     " "
	CALL      SND_CHAR
	RETURN

;ROUTINE TO RECALL CONTENTS OF EEPROM MEMORY POINTED TO BY THE KEY HIT
;AND DISPLAY ON LCD

RCLMEM                                                                        
	BCF       STATUS,RP0
	MOVLW     D'10'               ;FIRST CHECK LOCATION IS OK
	SUBWF     LASTKEY,W
	BTFSC     CARRY_BIT           ;SKIP IF NEGATIVE RESULT
	GOTO      NODIGIT             ;NOT A VALID DIGIT KEY
	MOVF      LASTKEY,W           ;VALID SO PRINT
	CALL      BCD_ASCII
	CALL      SND_CHAR
	BTFSC     KEYFLAG,4           ;1ST OR 2ND NUMBER
	GOTO      _SEC
	BSF       KEYFLAG,4           ;FLAG FIRST NUMBER
	MOVF      LASTKEY,W
	MOVWF     MEMHIGH             ;SAVE FIRST DIGIT OF MEM LOCATION
	CALL      MULT_X_10
	MOVWF     LOCATION            ;SAVE FIRST NUMBER
	RETURN
_SEC      BCF       KEYFLAG,4
	CALL      PAUSE
	MOVF      LASTKEY,W
	MOVWF     MEMLOW              ;SAVE SECOND DIGIT OF MEM LOCATION
	ADDWF     LOCATION,F          ;ADD LAST TWO NUMBERS ENTERED
	BCF       CARRY_BIT
	RLF       LOCATION,W          ;DOUBLE IT INTO W
	MOVWF     ADDR                ;TO GET EEPROM ADDRESS
READMEM
          CLRF      EMPTY
          BSF       PCLATH,3
          BTFSC     RIT,0
          CALL      DESEL_RIT
          BCF       PCLATH,3
	CALL      READ                ;GET DATA IN NEXT 2 ADDRESSES
	MOVLW     B'10000000'         ;DD RAM ADRESS TO ZERO
	CALL      SND_CMD   
	MOVF      HWORDH,W
	MOVWF     TEMP
	INCF      TEMP,W
	BTFSC     ZERO_BIT            ;BLANK ADDRESS IF SET
	GOTO      PR_DEFLT
	MOVF      HWORDH,W
	MOVWF     TEMP
	SWAPF     TEMP,W
	ANDLW     B'00001111'         ;FIRST DIGIT 10MHz NOW IN W
	BTFSS     ZERO_BIT
	GOTO      PR_DIG
PR_SPC    MOVLW     " "
	GOTO      PR_SP
PR_DIG    CALL      BCD_ASC
PR_SP     CALL      SND_CHAR            ;PRINT 10MHz
	MOVF      HWORDH,W
	ANDLW     B'00001111'         ;1MHz NOW IN W
	CALL      BCD_ASC
	CALL      SND_CHAR            ;PRINT 1MHz
	MOVLW     "."
	CALL      SND_CHAR            ;PRINT DOT
	MOVF      HWORDL,W
	MOVWF     TEMP
	SWAPF     TEMP,W
	ANDLW     B'00001111'         ;100KHz NOW IN W
	CALL      BCD_ASC
	CALL      SND_CHAR            ;PRINT 100KHZ
	MOVF      HWORDL,W
	ANDLW     B'00001111'         ;10KHz NOW IN W
	CALL      BCD_ASC        
	CALL      SND_CHAR            ;PRINT 10KHZ
	MOVF      LWORDH,W
	MOVWF     TEMP
	SWAPF     TEMP,W
	ANDLW     B'00001111'         ;1KHZ DIGIT NOW IN W
	CALL      BCD_ASC
	CALL      SND_CHAR            ;PRINT 1KHz
	MOVF      LWORDH,W
	ANDLW     B'00001111'         ;100HZ NOW IN W
	CALL      BCD_ASC          
	CALL      SND_CHAR            ;PRINT 100HZ
	MOVF      LWORDL,W
	MOVWF     TEMP
	SWAPF     TEMP,W
	ANDLW     B'00001111'         ;10HZ NOW IN W
	CALL      BCD_ASC
	CALL      SND_CHAR            ;PRINT 10HZ
	MOVLW     "0"
	CALL      SND_CHAR            ;PRINT "0" FOR 1HZ DIGIT
PR_MHz    MOVLW     " "
	CALL      SND_CHAR
	MOVLW     "M"
	CALL      SND_CHAR
	MOVLW     "H"
	CALL      SND_CHAR
	MOVLW     "z"
	CALL      SND_CHAR
	MOVLW     " "
	CALL      SND_CHAR
	BTFSC     MEMLOW,7            ;PRINT MEM LOC?
	GOTO      CLR_MEM_LOC         ;CLEAR MEMORY LOCATION
	MOVF      MEMHIGH,W           ;PRINT THE MEMORY LOCATION
	CALL      BCD_ASC
	CALL      SND_CHAR
	MOVF      MEMLOW,W
	CALL      BCD_ASC
	CALL      SND_CHAR
	GOTO      GET_MODE
CLR_MEM_LOC
	MOVLW     " "
	CALL      SND_CHAR
	CALL      SND_CHAR
GET_MODE
	MOVF      LWORDL,W
          ANDLW     B'00001111'         ;MODE NOW IN W
	MOVWF     MODE
	BTFSC     SIGGEN,0
	CLRF      MODE
	RETURN

BCD_ASC
	MOVWF     TEMP
	BSF       TEMP,4
	BSF       TEMP,5
	MOVF      TEMP,W
	RETURN

PR_DEFLT
          MOVLW     D'200'              ;PRINT 7.03MHz ON FIRST POWER UP
          SUBWF     ADDR,W
          BTFSC     ZERO_BIT
          GOTO      DISP_DFLT           ;ELSE DISPLAY EMPTY
          MOVLW     D'202'              ;PRINT 7.03MHz ON FIRST SIGGEN
          SUBWF     ADDR,W
          BTFSS     ZERO_BIT
          GOTO      DISP_EMPTY
DISP_DFLT
          MOVLW     " "                
	CALL      SND_CHAR        
          MOVLW     "7"
	CALL      SND_CHAR
	MOVLW     "."
	CALL      SND_CHAR
          MOVLW     "0"
	CALL      SND_CHAR
          MOVLW     "3"
	CALL      SND_CHAR
          MOVLW     "0"
	CALL      SND_CHAR
	CALL      SND_CHAR           
	CALL      SND_CHAR 
	CALL      SND_CHAR
	MOVLW     B'00001000'
	MOVWF     LWORDL              ;DEFAULT TO CW
	GOTO      PR_MHz

DISP_EMPTY
          BTFSC     SCAN,0              ;DONT FLASH "EMPTY" IF SCANNING
          GOTO      FLAGEMPTY
          MOVLW     B'10010111'         ;LCD ADDR 17
	CALL      SND_CMD
	CALL      REC_CHAR
	MOVWF     MEMHIGH
	CALL      REC_CHAR
	MOVWF     MEMLOW
          CALL      CLRPAGE2              ;WIPE PAGE 2
	CALL      SETPAGE2
	MOVLW     B'00001100'         ;CURSOR OFF NO BLINK
	CALL      SND_CMD
	MOVF      MEMHIGH,W
	CALL      SND_CHAR
	MOVF      MEMLOW,W
	CALL      SND_CHAR
          MOVLW     " "
          CALL      SND_CHAR
          MOVLW     "E"
          CALL      SND_CHAR
          MOVLW     "m"
          CALL      SND_CHAR
          MOVLW     "p"
          CALL      SND_CHAR
          MOVLW     "t"
          CALL      SND_CHAR
          MOVLW     "y"
          CALL      SND_CHAR
	CALL      PAUSE
	CALL      PAUSE
          MOVLW     B'10010000'         ;LCD ADDR 10
	CALL      SND_CMD
          MOVLW     8
          MOVWF     COUNT
	MOVLW     " "
BNK       CALL      SND_CHAR
          DECFSZ    COUNT,F
          GOTO      BNK
	CALL      PAUSE
	CALL      PAUSE
          MOVLW     B'10010000'         ;LCD ADDR 10
	CALL      SND_CMD
	MOVF      MEMHIGH,W
	CALL      SND_CHAR
	MOVF      MEMLOW,W
	CALL      SND_CHAR
          MOVLW     " "
          CALL      SND_CHAR
          MOVLW     "E"
          CALL      SND_CHAR
          MOVLW     "m"
          CALL      SND_CHAR
          MOVLW     "p"
          CALL      SND_CHAR
          MOVLW     "t"
          CALL      SND_CHAR
          MOVLW     "y"
          CALL      SND_CHAR
	CALL      PAUSE
	CALL      PAUSE
	CALL      PAUSE
	CALL      PAUSE
	CALL      PAUSE
          RETURN          
FLAGEMPTY
          BSF       EMPTY,0
          RETURN

;ROUTINE TO SERVICE AND FLAG A STORE KEY HIT

STORE
	BCF       STATUS,RP0
	CLRF      KEYFLAG
	BSF       KEYFLAG,2           ;FLAG STORE KEY PRESSED        
	CALL      SETPAGE2
	MOVLW     "S"
	CALL      SND_CHAR
	MOVLW     "t"
	CALL      SND_CHAR
	MOVLW     "o"
	CALL      SND_CHAR
	MOVLW     "r"
	CALL      SND_CHAR
	MOVLW     "e"
	CALL      SND_CHAR
	MOVLW     " "
	CALL      SND_CHAR
	RETURN

;ROUTINE TO STORE THE CURRENT DISPLAY INTO THE MEMORY LOCATION IN LASTKEY

STOREDISPLAY

	MOVLW     D'10'               ;FIRST CHECK LOCATION IS OK
	SUBWF     LASTKEY,W
	BTFSC     CARRY_BIT           ;SKIP IF NEGATIVE RESULT
	GOTO      NODIGIT             ;NOT A VALID DIGIT KEY
	MOVF      LASTKEY,W           ;VALID SO PRINT
	CALL      BCD_ASCII
	CALL      SND_CHAR
	BTFSC     KEYFLAG,4           ;1ST OR 2ND NUMBER
	GOTO      SEC
	BSF       KEYFLAG,4           ;FLAG FIRST NUMBER
	MOVF      LASTKEY,W
	CALL      MULT_X_10           ;MULTIPLY BY 10
	MOVWF     LOCATION            ;SAVE FIRST NUMBER
	RETURN
SEC
          CALL      FLASHD

STOREDISP  
	BSF       PCLATH,3
	CALL      DISP_2_BCD          ;CONVERT DISPLAY TO 4 BYTES
	BCF       PCLATH,3
	CALL      STOREMEM
NODIGIT   CALL      CLRPAGE2
	RETURN

;STORE HWORDH, HWORDL, LWORDH AND LWORDL INTO ADDR AND ADDR+1

STOREMEM          
	CALL      EWEN
	MOVF      HWORDH,W
	MOVWF     MSD
	MOVF      HWORDL,W
	MOVWF     LSD
	CALL      WRITE
	INCF      ADDR,F
	MOVF      LWORDH,W
	MOVWF     MSD
	MOVF      LWORDL,W
	MOVWF     LSD
	CALL      WRITE
	CALL      EWDS
	RETURN

;ROUTINE TO SERVICE AND FLAG A CLEAR KEY HIT

CLEAR
	BCF       STATUS,RP0
	CLRF      KEYFLAG
          BSF       KEYFLAG,5           ;FLAG CLR KEY PRESSED        
	CALL      SETPAGE2
          MOVLW     "E"
	CALL      SND_CHAR
          MOVLW     "r"
	CALL      SND_CHAR
          MOVLW     "a"
	CALL      SND_CHAR
          MOVLW     "s"
	CALL      SND_CHAR
	MOVLW     "e"
	CALL      SND_CHAR
	MOVLW     " "
	CALL      SND_CHAR
	RETURN

ERASEMEM
	BCF       STATUS,RP0
	MOVLW     D'10'               ;FIRST CHECK LOCATION IS OK
	SUBWF     LASTKEY,W
	BTFSC     CARRY_BIT           ;SKIP IF NEGATIVE RESULT
	GOTO      NODIGIT             ;NOT A VALID DIGIT KEY
	MOVF      LASTKEY,W           ;VALID SO PRINT
	CALL      BCD_ASCII
	CALL      SND_CHAR
	BTFSC     KEYFLAG,4           ;1ST OR 2ND NUMBER
          GOTO      SECOND
	BSF       KEYFLAG,4           ;FLAG FIRST NUMBER
	MOVF      LASTKEY,W
	CALL      MULT_X_10           ;MULTIPLY BY 10
	MOVWF     LOCATION            ;SAVE FIRST NUMBER
	RETURN
SECOND
          CALL      FLASHD
          CALL      EWEN
          CALL      ERASE
          CALL      EWDS
	MOVLW     B'10001110'         ;LCD ADDR 0E
	CALL      SND_CMD
	MOVLW     " "                 ;BLANK OFF ANY MEMORY LOCATION
	CALL      SND_CHAR
	MOVLW     " "
	CALL      SND_CHAR
          CALL      CLRPAGE2
          RETURN

FLASHD
	MOVLW     B'00001100'         ;CURSOR OFF
	CALL      SND_CMD
	MOVLW     B'10010110'         ;LCD ADDR 16
	CALL      SND_CMD
	CALL      REC_CHAR
	MOVWF     MEMHIGH
	CALL      REC_CHAR
	MOVWF     MEMLOW
	MOVLW     B'10010101'         ;LCD ADDR 15
	CALL      SND_CMD
	MOVLW     "d"
	CALL      SND_CHAR
	MOVLW     " "
	CALL      SND_CHAR
	MOVF      MEMHIGH,W
	CALL      SND_CHAR
	MOVF      MEMLOW,W
	CALL      SND_CHAR
	CALL      PAUSE
	CALL      PAUSE
	MOVLW     B'10010111'         ;LCD ADDR 17
	CALL      SND_CMD
	MOVLW     " "
	CALL      SND_CHAR
	CALL      SND_CHAR
	CALL      PAUSE
	CALL      PAUSE
	MOVLW     B'10010111'         ;LCD ADDR 17
	CALL      SND_CMD
	MOVF      MEMHIGH,W
	CALL      SND_CHAR
	MOVF      MEMLOW,W
	CALL      SND_CHAR
	CALL      PAUSE
	CALL      PAUSE
	CALL      PAUSE
	CALL      PAUSE
	CALL      PAUSE
	MOVF      LASTKEY,W
	ADDWF     LOCATION,F          ;ADD LAST TWO NUMBERS ENTERED
	BCF       CARRY_BIT
	RLF       LOCATION,W          ;DOUBLE IT INTO W
	MOVWF     ADDR                ;TO GET EEPROM ADDRESS
          RETURN

;ROUTINE THAT CHECKS THE FIRST TWO DIGITS OF DISPLAY AND CONVERTS TO BAND
;RETURNS WITH BAND IN W (0 - 9) OR 10 IF NOT IN AMATEUR BAND

CONV_BAND
	BTFSC     SIGGEN,0
	RETLW     D'10'
	CALL      REC_CHAR            ;FIRST CHARACTER IN W
	MOVWF     TEMP
	MOVLW     " "
	SUBWF     TEMP,W
	BTFSC     ZERO_BIT            ;SPACE (0) IF SET
	GOTO      ZERO_SEC
	MOVLW     "1"                 
	SUBWF     TEMP,W
	BTFSC     ZERO_BIT
	GOTO      ONE_SEC
	MOVLW     "2"
	SUBWF     TEMP,W
	BTFSC     ZERO_BIT
	GOTO      TWO_SEC
	RETLW     D'10'               ;NOT AMATEUR BAND
ZERO_SEC  CALL      REC_CHAR            ;SND DIGIT IN W
	MOVWF     TEMP
	MOVLW     "1"
	SUBWF     TEMP,W
	BTFSC     ZERO_BIT
	RETLW     0                   ;TOP BAND
	MOVLW     "3"
	SUBWF     TEMP,W
	BTFSC     ZERO_BIT
	RETLW     1                   ;3.5 MHz
	MOVLW     "7"
	SUBWF     TEMP,W
	BTFSC     ZERO_BIT
	RETLW     2                   ;7 MHz
	RETLW     D'10'               ;NOT AMATEUR BAND
ONE_SEC   CALL      REC_CHAR
	MOVWF     TEMP
	MOVLW     "0"
	SUBWF     TEMP,W
	BTFSC     ZERO_BIT
	RETLW     3                   ;10 MHz
	MOVLW     "4"
	SUBWF     TEMP,W
	BTFSC     ZERO_BIT
	RETLW     4                   ;14 MHZ
	MOVLW     "8"
	SUBWF     TEMP,W
	BTFSC     ZERO_BIT
	RETLW     5                   ;18 MHz
	RETLW     D'10'               ;NOT AMATEUR BAND
TWO_SEC   CALL      REC_CHAR 
	MOVWF     TEMP
	MOVLW     "1" 
	SUBWF     TEMP,W
	BTFSC     ZERO_BIT
	RETLW     6                   ;21 MHz
	MOVLW     "4"    
	SUBWF     TEMP,W
	BTFSC     ZERO_BIT
	RETLW     7                   ;24 MHz
	MOVLW     "8"
	SUBWF     TEMP,W
	BTFSC     ZERO_BIT
	RETLW     8                   ;28 MHZ
	MOVLW     "9"
	SUBWF     TEMP,W
	BTFSC     ZERO_BIT
	RETLW     9                   ;29 MHZ
	RETLW     D'10'               ;NOT AMATEUR BAND

;ROUTINE TO SET THE BFO OFFSET DIRECTION.

SET_BFO_DIR
          MOVF      CFGBFO_1,W          ;GET BFO CONFIG INFO
	MOVWF     BAND1
          MOVF      CFGBFO_2,W
	MOVWF     BAND2
	MOVF      BAND,W
          MOVWF     COUNT1              ;BAND CODE NOW IN COUNT1
	BTFSC     COUNT1,3            ;8 OR 9 IF SET
	GOTO      HI_BND
	INCF      COUNT1,F
ROTAT     RLF       BAND1,F
	DECFSZ    COUNT1,F
          GOTO      ROTAT
	BTFSC     CARRY_BIT
          GOTO      SETDFLT
          BCF       BFOSIGN,0           ;REVERSE DIRECTION
	RETURN
SETDFLT   BSF       BFOSIGN,0           ;DEFAULT DIRECTION
	RETURN
HI_BND
	MOVLW     7
	SUBWF     COUNT1,F            ;8 OR 9 BECOMES 1 OR 2
	MOVF      BAND2,W
	MOVWF     BAND1
          GOTO      ROTAT

;ROUTINE TO SERVICE A CLR KEY HIT

CLRPAGE2
	BCF       STATUS,RP0
	MOVLW     B'00000010'         ;RETURN HOME
	CALL      SND_CMD   
	MOVLW     B'00001100'         ;CURSOR OFF
	CALL      SND_CMD   
	MOVLW     B'10010000'         ;SET LCD ADDR H10
	CALL      SND_CMD   
	MOVLW     D'24'
	MOVWF     COUNT
BLANK1    MOVLW     " "
	CALL      SND_CHAR
	DECFSZ    COUNT,F
	GOTO      BLANK1
	MOVLW     B'11010000'         ;SET LCD ADDR H50
	CALL      SND_CMD   
	MOVLW     D'24'
	MOVWF     COUNT
BLANK2    MOVLW     " "
	CALL      SND_CHAR
	DECFSZ    COUNT,F
	GOTO      BLANK2
	MOVLW     B'10000000'         ;LCD ADDR 0
	CALL      SND_CMD   
	CLRF      KEYFLAG
	CLRF      FNCNO
	CLRF      ENTRY_COUNT
	RETURN

;ROUTINE TO SET THE DISPLAY TO PAGE 2

SETPAGE2
	BCF       STATUS,RP0
	MOVLW     D'16'               ;16 SHIFTS
	MOVWF     COUNT
SHIFTL    MOVLW     B'00011000'         ;SHIFT DISPLAY LEFT
	CALL      SND_CMD   
	DECFSZ    COUNT,F
	GOTO      SHIFTL
	MOVLW     B'10010000'         ;LCD ADDR H10
	CALL      SND_CMD   
	MOVLW     B'00001111'         ;BLINKING CURSOR
	CALL      SND_CMD   
	RETURN

;ROUTINE TO MULTIPLY W BY 10

MULT_X_10
	MOVWF     ACCU         
	MOVLW     9
	MOVWF     COUNT
	MOVF      ACCU,W
AGAIN     ADDWF     ACCU,F
	DECFSZ    COUNT,F
	GOTO      AGAIN
	MOVF      ACCU,W
	RETURN


;SERIAL EEPROM ROUTINES

;START BIT ROUTINE

BSTART
	BCF       STATUS,RP0
	BCF       DI
	BCF       CS
	BCF       CLK
	NOP
	BSF       CS
	BSF       DI
	NOP
	BSF       CLK
	NOP
	NOP
	BCF       CLK
	RETURN

;ROUTINE TO READ ONE BIT OF DATA AND STORE IT IN 'DIN'

BITIN
	BCF       STATUS,RP0
	BSF       DIN
	BSF       CLK
	NOP
	BTFSS     DO
	BCF       DIN
	BCF       CLK
	RETURN

;ROUTINE TO READ ONE BYTE OF DATA INTO THE 'DATAI' REGISTER

RX
	BCF       STATUS,RP0
	CLRF      DATAI
	MOVLW     8
	MOVWF     COUNT
	BCF       CARRY_BIT
RXLP      RLF       DATAI,F
	CALL      BITIN
	BCF       DATAI,0
	BTFSC     DIN     
	BSF       DATAI,0
	DECFSZ    COUNT,F
	GOTO      RXLP
	RETURN

;ROUTINE TO SEND ONE BIT OF DATA FROM 'DOUT' TO EEPROM

BITOUT
	BCF       STATUS,RP0
	BTFSS     DOUT
	GOTO      BITLOW
	BSF       DI
	GOTO      CLKOUT
BITLOW    BCF       DI
CLKOUT    BSF       CLK
	NOP
	BCF       CLK
	RETURN

;ROUTINE TO TRANSMIT A BYTE OF DATA FROM 'DATAO' TO EEPROM

TX
	BCF       STATUS,RP0
	MOVF      BITS,W
	MOVWF     COUNT
TXLP      BCF       DOUT
	BTFSC     TXBUF,7
	BSF       DOUT
	CALL      BITOUT
	RLF       TXBUF,F
	DECFSZ    COUNT,F
	GOTO      TXLP
	RETURN

;ROUTINE TO ENABLE ERASE AND WRITE TO THE EEPROM

EWEN
	BCF       STATUS,RP0
	CALL      BSTART
	MOVLW     2
	MOVWF     BITS
	MOVLW     B'00000000'         ;OPCODE FOR EWEN
	MOVWF     TXBUF
	CALL      TX                  ;TRANSMIT IT
	MOVLW     8
	MOVWF     BITS
	MOVLW     B'11000000'         ;ADDRESS TO FOLLOW EWEN
	MOVWF     TXBUF
	CALL      TX
	BCF       CS
	CALL      DEBOUNCE            ;WAIT 20mS
	RETURN

;ROUTINE TO DISABLE WRITES TO THE EEPROM

EWDS
	BCF       STATUS,RP0
	CALL      BSTART
	MOVLW     2
	MOVWF     BITS
	MOVLW     B'00000000'         ;OPCODE FOR EWDS
	MOVWF     TXBUF
	CALL      TX                  ;TRANSMIT IT
	MOVLW     8
	MOVWF     BITS
	MOVLW     B'00000000'         ;ADDRESS TO FOLLOW EWDS
	MOVWF     TXBUF
	CALL      TX
	BCF       CS
	CALL      DEBOUNCE            ;WAIT 20mS
	RETURN

;ROUTINE TO ERASE THE CONTENTS OF 'ADDR' OF THE EEPROM

ERASE
	CALL      BSTART              ;1 ENTERED     
	MOVLW     2
	MOVWF     BITS
          MOVLW     B'11111111'         ;OPCODE FOR ERASE
          MOVWF     TXBUF
	CALL      TX                  ;TRANSMIT IT
	MOVLW     8
	MOVWF     BITS
          MOVF      ADDR,W              ;ADDRESS TO ERASE
	MOVWF     TXBUF
	CALL      TX
	BCF       CS
	CALL      DEBOUNCE            ;WAIT 20mS
	RETURN

;ROUTINE TO STORE THE CONTENTS OF MSD AND LSD TO ADDRESS 'ADDR' OF THE EEPROM

WRITE
	BCF       STATUS,RP0
	CALL      BSTART
	MOVLW     2
	MOVWF     BITS
	MOVLW     B'01000000'         ;OPCODE FOR WRITE
	MOVWF     TXBUF
	CALL      TX                  ;TRANSMIT IT
	MOVLW     8
	MOVWF     BITS
	MOVF      ADDR,W              ;ADDRESS TO WRITE TO
	MOVWF     TXBUF
	CALL      TX                  ;TRANSMIT IT
	MOVLW     8
	MOVWF     BITS
	MOVF      MSD,W               ;HIGH BYTE TO TRANSMIT
	MOVWF     TXBUF
	CALL      TX
	MOVF      LSD,W
	MOVWF     TXBUF
	CALL      TX
	BCF       CS
	CALL      DEBOUNCE
	RETURN

;ROUTINE TO READ THE DATA STORED IN THE EEPROM AT ADDRESS 'ADDR' INTO
;HWORDH AND HWORDL AND THE DATA STORED AT ADDR+1 INTO LWORDH AND LWORDL

READ
	BCF       STATUS,RP0
	CALL      BSTART
	MOVLW     2
	MOVWF     BITS
	MOVLW     B'10000000'         ;OPCODE FOR READ (B'10')
	MOVWF     TXBUF
	CALL      TX
	MOVLW     8
	MOVWF     BITS
	MOVF      ADDR,W
	MOVWF     TXBUF
	CALL      TX
	CALL      RX
	MOVF      DATAI,W
	MOVWF     HWORDH
	CALL      RX
	MOVF      DATAI,W
	MOVWF     HWORDL
	CALL      RX
	MOVF      DATAI,W
	MOVWF     LWORDH
	CALL      RX
	MOVF      DATAI,W
	MOVWF     LWORDL
	BCF       CS
	RETURN

;STORE CONFIG DATA IN CFGBND_1, CFGBND_2, CFGDIR_1 AND CFGDIR_2 INTO
;EEPROM LOCATION 204 AND 205 AND CONFIG DATA IN CFGBFO_1 AND CFGBFO_2,
;STEP AND SCANDELAY INTO EEPROM LOCATION 206 AND 207

STO_CONFIG
	MOVLW     D'204'
	MOVWF     ADDR
	MOVF      CFGBND_1,W
	MOVWF     HWORDH
	MOVF      CFGBND_2,W
	MOVWF     HWORDL
	MOVF      CFGDIR_1,W
	MOVWF     LWORDH
	MOVF      CFGDIR_2,W
	MOVWF     LWORDL
	CALL      STOREMEM
          MOVLW     D'206'
          MOVWF     ADDR
          MOVF      CFGBFO_1,W
          MOVWF     HWORDH
          MOVF      CFGBFO_2,W
          MOVWF     HWORDL
          MOVF      STEP,W
          MOVWF     LWORDH
          MOVF      SCANDELAY,W
          MOVWF     LWORDL
          CALL      STOREMEM
	RETURN

;ROUTINE TO DISPLAY THE SELECTED SCREEN

DISPFNC                          
	BCF       STATUS,RP0
	MOVF      LASTKEY,W
	CALL      BCD_ASCII        
	CALL      SND_CHAR            ;PRINT FUNCTION NUMBER
	CALL      PAUSE
	MOVF      LASTKEY,W
	BTFSC     ZERO_BIT            ;NO FUNCTION 0
          GOTO      CLRALL
          BSF       PCLATH,3
	MOVLW     1
	SUBWF     LASTKEY,W           ;1?
	BTFSC     ZERO_BIT
	GOTO      FUNC_1
	MOVLW     2
	SUBWF     LASTKEY,W           ;2?
	BTFSC     ZERO_BIT       
	GOTO      FUNC_2
	MOVLW     3
	SUBWF     LASTKEY,W           ;3?
	BTFSC     ZERO_BIT
	GOTO      FUNC_3
	MOVLW     4
	SUBWF     LASTKEY,W           ;4?
	BTFSC     ZERO_BIT
	GOTO      FUNC_4
	MOVLW     5
	SUBWF     LASTKEY,W           ;5?
	BTFSC     ZERO_BIT
	GOTO      FUNC_5
	MOVLW     6
	SUBWF     LASTKEY,W           ;6?
	BTFSC     ZERO_BIT
	GOTO      FUNC_6
	MOVLW     7
	SUBWF     LASTKEY,W           ;7?
	BTFSC     ZERO_BIT
	GOTO      FUNC_7
	MOVLW     8
	SUBWF     LASTKEY,W           ;8?
	BTFSC     ZERO_BIT
	GOTO      FUNC_8
	MOVLW     9
	SUBWF     LASTKEY,W           ;9?
	BTFSC     ZERO_BIT
	GOTO      FUNC_9
	MOVLW     D'10'
	SUBWF     LASTKEY,W           ;.?
	BTFSC     ZERO_BIT
          GOTO      FUNC_DOT
          MOVLW     D'13'
          SUBWF     LASTKEY,W           ;STO?
	BTFSC     ZERO_BIT
          GOTO      FUNC_STO
          MOVLW     D'14'
          SUBWF     LASTKEY,W           ;RCL?
	BTFSC     ZERO_BIT
          GOTO      FUNC_RCL
          BCF       PCLATH,3
          GOTO      CLRALL          

FUNC_RCL2   
          CALL      CLRPAGE2              ;WIPE PAGE 2
	CALL      SETPAGE2
          MOVLW     "S"
	CALL      SND_CHAR
          MOVLW     "c"
	CALL      SND_CHAR 
          MOVLW     "a"
	CALL      SND_CHAR
          MOVLW     "n"
	CALL      SND_CHAR
          MOVLW     " "
	CALL      SND_CHAR 
          MOVLW     "d"
	CALL      SND_CHAR
          MOVLW     "e"
	CALL      SND_CHAR
          MOVLW     "l"
	CALL      SND_CHAR 
          MOVLW     "a"
	CALL      SND_CHAR
          MOVLW     "y"
	CALL      SND_CHAR
          MOVLW     B'11010000'         ;LCD ADDR 50
	CALL      SND_CMD   
          MOVLW     "x"
	CALL      SND_CHAR 
          MOVLW     "0"
	CALL      SND_CHAR 
          MOVLW     "."
	CALL      SND_CHAR
          MOVLW     "5"
	CALL      SND_CHAR
          MOVLW     " "
	CALL      SND_CHAR
          MOVLW     "s"
	CALL      SND_CHAR
          MOVLW     "e"
	CALL      SND_CHAR 
          MOVLW     "c"
	CALL      SND_CHAR 
          MOVLW     "o"
	CALL      SND_CHAR
          MOVLW     "n"
	CALL      SND_CHAR
          MOVLW     "d"
	CALL      SND_CHAR 
          MOVLW     "s"
	CALL      SND_CHAR
	MOVLW     B'10011111'         ;LCD ADDR 1F
	CALL      SND_CMD   
	MOVF      LASTKEY,W         
	MOVWF     FNCNO               ;FUNCTION NUMBER    
	RETURN

FUNCRCL2
	BCF       STATUS,RP0
	MOVF      LASTKEY,W
	CALL      BCD_ASCII
	CALL      SND_CHAR            ;PRINT CHOICE
	CALL      PAUSE
	MOVLW     D'10'
	SUBWF     LASTKEY,W         
	BTFSC     CARRY_BIT           ;CHECK FOR GREATER THAN 9 ENTERED
          RETURN
	MOVF      LASTKEY,W
          MOVWF     SCANDELAY
          CALL      STO_CONFIG
          RETURN

;*******************************************************************
          ORG       H'0800'
;*******************************************************************

;ROUTINE TO SEND THE BAND AND MODE TO THE TWO 595'S

SND_BAND
	CLRF      BAND1
	CLRF      BAND2
	BTFSC     SIGGEN,0
	GOTO      SND_SERIAL
	MOVLW     B'10000000'         ;LCD ADDR 0
	BCF       PCLATH,3
	CALL      SND_CMD
	BCF       PCLATH,3 
	CALL      CONV_BAND           ;GET BAND CODE INTO W
	BSF       PCLATH,3
          MOVWF     BAND          
	MOVLW     D'10'               ;CHECK FOR NON AMATEUR BAND
          SUBWF     BAND,W              ;AND CLEAR BAND SWITCHES IF SO
	BTFSC     ZERO_BIT
	GOTO      SND_SERIAL
          BTFSC     BAND,3              ;GREATER THAN 7 IF SET
	GOTO      TENMTRS
          MOVF      BAND,W
	BCF       PCLATH,3 
	CALL      BCODE_SER           ;CONVERT TO SERIAL BYTE
	BSF       PCLATH,3 
	MOVWF     BAND1
	GOTO      GETMODE
TENMTRS   MOVF      BAND,W
	BCF       PCLATH,3
	CALL      BCODE_SER           ;CONVERT TO SERIAL BYTE
	BSF       PCLATH,3
	MOVWF     BAND2
GETMODE   MOVLW     1                   ;FM?
          SUBWF     MODE,W
          BTFSC     ZERO_BIT
          GOTO      SET_FM
          MOVLW     2                   ;AM
          SUBWF     MODE,W
          BTFSC     ZERO_BIT
          GOTO      SET_AM
          MOVLW     3                   ;CW
          SUBWF     MODE,W
          BTFSC     ZERO_BIT
          GOTO      SET_CW
          MOVLW     4                   ;LSB
          SUBWF     MODE,W
          BTFSC     ZERO_BIT
          GOTO      SET_LSB
SET_USB   BSF       BAND2,3
          GOTO      SND_SERIAL
SET_LSB   BSF       BAND2,4
          GOTO      SND_SERIAL
SET_CW    BSF       BAND2,5
          GOTO      SND_SERIAL
SET_AM    BSF       BAND2,6
          GOTO      SND_SERIAL
SET_FM    BSF       BAND2,7
SND_SERIAL
	MOVF      BAND1,W
          MOVWF     TEMP
	CALL      SND_595  
	BSF       LATCH1
	BCF       LATCH1
	MOVF      BAND2,W
          MOVWF     TEMP
	CALL      SND_595
	BCF       LATCH2
	BSF       LATCH2
	RETURN
SND_595
	MOVLW     8
	MOVWF     COUNT
	BCF       SCLK      
BIT       BCF       A
          RLF       TEMP,F
	BTFSC     CARRY_BIT
	BSF       A
	BSF       SCLK
	BCF       SCLK
	DECFSZ    COUNT,F
	GOTO      BIT
	RETURN


;ROUTINE TO READ THE DISPLAY AND PROGRAMME THE DDS WITH THE FREQUENCY

SND_DDS
	BTFSC     RIT,0
	GOTO      SND_RIT
	CALL      SAV_FREQ            ;SAVE DISPLAYED FREQ IN RAM
          CALL      SET_MHz             ;5MHz (5.5 OR 6 IF REV) OR ZERO
	CALL      CLRACCU
	BTFSS     SIGGEN,0
	GOTO      JUSTKHZ
	CALL      MULT10MHz
	CALL      MULT1MHz
JUSTKHZ   CALL      MULT100KHz
	CALL      MULT10KHz
	CALL      MULT1KHz
	CALL      MULT100Hz
	CALL      MULT10Hz
	CALL      MULT1Hz
	BTFSC     REVERSE,0
	CALL      NEGATE
	CALL      ADDTOMHz
          CALL      ADD_BFO
	CALL      PROG_DDS_1          ;RX FREQ  
	CALL      FR1_TO_FR0          ;RX FREQ TO TX FREQ
	CALL      PROG_DDS_0          ;TX FREQ   
	RETURN

SND_RIT
	CALL      FR0_TO_FR1          ;TX FREQ TO RX FREQ
	CALL      SAV_RIT
	CALL      CLRACCU
	CALL      MULT1KHz
	CALL      MULT100Hz
	CALL      MULT10Hz
	CALL      MULT1Hz
          BTFSC     REVERSE,0
          GOTO      REV
NOR       BTFSC     RIT,7               ;NORMAL TUNING NOW CHECK POLARITY
          GOTO      NEG
          GOTO      POS
REV       BTFSC     RIT,7               ;REVERSE TUNING NOW CHECK POLARITY
          GOTO      POS
NEG       CALL      NEGATE
POS       CALL      ADDTOMHz
	CALL      PROG_DDS_1          ;RX FREQ
	RETURN

SAV_FREQ
	MOVLW     B'10000000'         ;LCD ADDR 00   
	BCF       PCLATH,3
	CALL      SND_CMD
	BSF       PCLATH,3
	MOVLW     9
	MOVWF     COUNT
	MOVLW     _10MHz
	MOVWF     FSR
GET_NEXT
	BCF       PCLATH,3  
	CALL      REC_CHAR            ;GET ASCII
	BSF       PCLATH,3
	MOVWF     TEMP
	MOVLW     H'0F'               ;CONVERT TO NUMBER
	ANDWF     TEMP,W
	MOVWF     INDF                ;SAVE IT
	INCF      FSR,F
	DECFSZ    COUNT,F
	GOTO      GET_NEXT
	RETURN

SAV_RIT
	MOVLW     B'11000101'         ;LCD ADDR 45   
	BCF       PCLATH,3      
	CALL      SND_CMD
	BSF       PCLATH,3
	MOVLW     4
	MOVWF     COUNT
	MOVLW     _1KHz
	MOVWF     FSR
GET_NXT
	BCF       PCLATH,3      
	CALL      REC_CHAR            ;GET ASCII
	BSF       PCLATH,3
	MOVWF     TEMP
	MOVLW     H'0F'               ;CONVERT TO NUMBER
	ANDWF     TEMP,W
	MOVWF     INDF                ;SAVE IT
	INCF      FSR,F
	DECFSZ    COUNT,F
	GOTO      GET_NXT
	RETURN

SET_MHz                               
	BTFSC     SIGGEN,0
	GOTO      SET_ZERO
	BTFSS     REVERSE,0
	GOTO      SET_5MHz
          MOVLW     2
          SUBWF     BAND,W
          BTFSS     CARRY_BIT           ;CARRY SET = 2 OR GREATER
          GOTO      SET_6MHz            ;0 OR 1 SO 6MHz
          MOVLW     7
          SUBWF     BAND,W
          BTFSS     CARRY_BIT           ;SET = BAND 7, 8 OR 9 SO 6MHz
          GOTO      SET_5_5MHz          ;ANY OTHER BAND = 5.5MHz
SET_6MHz                                
	MOVLW     H'1E'
	MOVWF     FREQ13
	MOVLW     H'B8'
	MOVWF     FREQ12
	MOVLW     H'51'
	MOVWF     FREQ11
	MOVLW     H'EB'
	MOVWF     FREQ10
	RETURN
SET_5_5MHz
          MOVLW     H'1C'
	MOVWF     FREQ13
          MOVLW     H'28'
	MOVWF     FREQ12
          MOVLW     H'F5'
	MOVWF     FREQ11
          MOVLW     H'C2'
	MOVWF     FREQ10
	RETURN
SET_5MHz
	MOVLW     H'19'
	MOVWF     FREQ13
	MOVLW     H'99'
	MOVWF     FREQ12
	MOVWF     FREQ11
	MOVWF     FREQ10
	RETURN
SET_ZERO
	CLRF      FREQ13
	CLRF      FREQ12
	CLRF      FREQ11
	CLRF      FREQ10
	RETURN
MULT10MHz
	MOVLW     H'33'               ;VALUE FOR 1 X 10MHz
	MOVWF     ACC0                ;IN ACC3 - ACC0
	MOVWF     ACC1
	MOVWF     ACC2
	MOVWF     ACC3
          MOVF      _10MHz,W            ;NO. OF 10MHz
	MOVWF     COUNT
	BTFSS     ZERO_BIT
	CALL      MULT_DIGIT
	RETURN
MULT1MHz
	MOVLW     H'52'               ;VALUE FOR 1 X 1MHz
	MOVWF     ACC0                ;IN ACC3 - ACC0
	MOVLW     H'B8'               
	MOVWF     ACC1
	MOVLW     H'1E'
	MOVWF     ACC2
	MOVLW     H'05'
	MOVWF     ACC3
	MOVF      _1MHz,W             ;NO. OF 1MHz
	MOVWF     COUNT
	BTFSS     ZERO_BIT
	CALL      MULT_DIGIT
	RETURN
MULT100KHz
	MOVLW     H'6E'               ;VALUE FOR 1 X 100KHz
	MOVWF     ACC0                ;IN ACC3 - ACC0
	MOVLW     H'12'               
	MOVWF     ACC1
	MOVLW     H'83'
	MOVWF     ACC2
	CLRF      ACC3
	MOVF      _100KHz,W           ;NO. OF 100KHz
	MOVWF     COUNT
	BTFSS     ZERO_BIT
	CALL      MULT_DIGIT
	RETURN
MULT10KHz
	MOVLW     H'71'
	MOVWF     ACC0
	MOVLW     H'1B'
	MOVWF     ACC1
	MOVLW     H'0D'
	MOVWF     ACC2
	CLRF      ACC3
	MOVF      _10KHz,W
	MOVWF     COUNT
	BTFSS     ZERO_BIT
	CALL      MULT_DIGIT
	RETURN
MULT1KHz
	MOVLW     H'8B'
	MOVWF     ACC0
	MOVLW     H'4F'
	MOVWF     ACC1
	MOVLW     H'01'
	MOVWF     ACC2
	CLRF      ACC3
	MOVF      _1KHz,W
	MOVWF     COUNT
	BTFSS     ZERO_BIT
	CALL      MULT_DIGIT
	RETURN
MULT100Hz
	MOVLW     H'8D'
	MOVWF     ACC0
	MOVLW     H'21'
	MOVWF     ACC1
	CLRF      ACC2
	CLRF      ACC3
	MOVF      _100Hz,W
	MOVWF     COUNT
	BTFSS     ZERO_BIT
	CALL      MULT_DIGIT
	RETURN
MULT10Hz
	MOVLW     H'5B'
	MOVWF     ACC0
	MOVLW     H'03'
	MOVWF     ACC1
	CLRF      ACC2
	CLRF      ACC3
	MOVF      _10Hz,W
	MOVWF     COUNT
	BTFSS     ZERO_BIT
	CALL      MULT_DIGIT
	RETURN
MULT1Hz
	MOVLW     H'56'
	MOVWF     ACC0
	CLRF      ACC1
	CLRF      ACC2
	CLRF      ACC3
	MOVF      _1Hz,W
	MOVWF     COUNT
	BTFSS     ZERO_BIT
	CALL      MULT_DIGIT
	RETURN

MULT_DIGIT     
	MOVF      ACC0,W
	ADDWF     ACCA,F
	BTFSS     CARRY_BIT
	GOTO      BYTE1
	MOVLW     1
	ADDWF     ACCB,F
	BTFSS     CARRY_BIT
	GOTO      BYTE1
	ADDWF     ACCC,F
	BTFSC     CARRY_BIT
	INCF      ACCD,F
BYTE1
	MOVF      ACC1,W
	ADDWF     ACCB,F
	BTFSS     CARRY_BIT
	GOTO      BYTE2
	MOVLW     1
	ADDWF     ACCC,F
	BTFSC     CARRY_BIT
	INCF      ACCD,F
BYTE2
	MOVF      ACC2,W
	ADDWF     ACCC,F
	BTFSC     CARRY_BIT
	INCF      ACCD,F
	MOVF      ACC3,W
	ADDWF     ACCD,F
	DECFSZ    COUNT,F
	GOTO      MULT_DIGIT
	RETURN
ADDTOMHz
	MOVF      ACCA,W
	ADDWF     FREQ10,F
	BTFSS     CARRY_BIT
	GOTO      _BYTE1
	MOVLW     1
	ADDWF     FREQ11,F
	BTFSS     CARRY_BIT
	GOTO      _BYTE1
	ADDWF     FREQ12,F
	BTFSC     CARRY_BIT
	INCF      FREQ13,F
_BYTE1
	MOVF      ACCB,W
	ADDWF     FREQ11,F
	BTFSS     CARRY_BIT
	GOTO      _BYTE2
	MOVLW     1
	ADDWF     FREQ12,F
	BTFSC     CARRY_BIT
	INCF      FREQ13,F
_BYTE2
	MOVF      ACCC,W
	ADDWF     FREQ12,F
	BTFSC     CARRY_BIT
	INCF      FREQ13,F
	MOVF      ACCD,W
	ADDWF     FREQ13,F
	RETURN

PROG_DDS_0
	BCF       _CS
	MOVF      FREQ03,W
	MOVWF     DATA_BUS
	CALL      STROBE
	MOVF      FREQ02,W
	MOVWF     DATA_BUS
	CALL      STROBE
	MOVF      FREQ01,W
	MOVWF     DATA_BUS
	CALL      STROBE
	MOVF      FREQ00,W
	MOVWF     DATA_BUS
	CALL      STROBE
	BSF       _CS
	MOVLW     H'08'               ;FREQ0 (00001000)
	MOVWF     DATA_BUS
	BSF       LOAD                ;TRANSFER TO FREQ0 (TX)
	BCF       LOAD
	RETURN

PROG_DDS_1
	BCF       _CS
	MOVF      FREQ13,W
	MOVWF     DATA_BUS
	CALL      STROBE
	MOVF      FREQ12,W
	MOVWF     DATA_BUS
	CALL      STROBE
	MOVF      FREQ11,W
	MOVWF     DATA_BUS
	CALL      STROBE
	MOVF      FREQ10,W
	MOVWF     DATA_BUS
	CALL      STROBE
	BSF       _CS
	MOVLW     H'09'               ;FREQ1 (00001001)
	MOVWF     DATA_BUS
	BSF       LOAD                ;TRANSFER TO FREQ1 (RX)
	BCF       LOAD
	RETURN

FR1_TO_FR0
	MOVF      FREQ10,W
	MOVWF     FREQ00
	MOVF      FREQ11,W
	MOVWF     FREQ01
	MOVF      FREQ12,W
	MOVWF     FREQ02
	MOVF      FREQ13,W
	MOVWF     FREQ03
	RETURN

FR0_TO_FR1
	MOVF      FREQ00,W
	MOVWF     FREQ10
	MOVF      FREQ01,W
	MOVWF     FREQ11
	MOVF      FREQ02,W
	MOVWF     FREQ12
	MOVF      FREQ03,W
	MOVWF     FREQ13
	RETURN        

CLRACCU
	CLRF      ACCA
	CLRF      ACCB
	CLRF      ACCC
	CLRF      ACCD
	RETURN

NEGATE
	COMF      ACCA,F
	COMF      ACCB,F               
	COMF      ACCC,F             
	COMF      ACCD,F               
	MOVLW     1
	ADDWF     ACCA,F
	BTFSS     CARRY_BIT
          RETURN
	ADDWF     ACCB,F
	BTFSS     CARRY_BIT
          RETURN
	ADDWF     ACCC,F
	BTFSS     CARRY_BIT
          RETURN
	INCF      ACCD,F
	RETURN

ADD_BFO
          BTFSS     CFGBFO_2,0          ;OFFSETS SELECTED?
          RETURN                        ;NO
          MOVLW     5
          SUBWF     MODE,W
          BTFSC     ZERO_BIT
          GOTO      SUB1500             ;USB
          MOVLW     4
          SUBWF     MODE,W
          BTFSC     ZERO_BIT
          GOTO      ADD1500             ;LSB
          MOVLW     3
          SUBWF     MODE,W
          BTFSC     ZERO_BIT
          GOTO      SUB700              ;CW
          RETURN                        ;AM, FM OR SIGGEN SO NO OFFSET
SUB1500
          MOVLW     H'51'
          MOVWF     ACCA
          MOVLW     H'F7'
          MOVWF     ACCB
	MOVLW     H'01'
          MOVWF     ACCC
          CLRF      ACCD
          BTFSC     BFOSIGN,0
          CALL      NEGATE
          CALL      ADDTOMHz
          RETURN
ADD1500          
          MOVLW     H'51'
          MOVWF     ACCA
          MOVLW     H'F7'
          MOVWF     ACCB
	MOVLW     H'01'
          MOVWF     ACCC
          CLRF      ACCD
          BTFSS     BFOSIGN,0
          CALL      NEGATE
          CALL      ADDTOMHz
          RETURN
SUB700
          BTFSS     CFGBFO_2,1          ;NO CW OFFSET IF CLEAR
          RETURN
          MOVLW     H'E1'
          MOVWF     ACCA
          MOVLW     H'EA'
          MOVWF     ACCB
          CLRF      ACCC
          CLRF      ACCD
          BTFSC     BFOSIGN,0
          CALL      NEGATE
          CALL      ADDTOMHz
          RETURN

STROBE
	BCF       _WR
	BSF       _WR
	RETURN

;ROUTINE TO SET THE TUNING DIRECTION. FOLLOWS A CALL TO CONV_BAND.
;BAND CODE IN BAND UPON ENTRY.

SET_DIR
	BTFSC     SIGGEN,0
	GOTO      SETFORW             ;ALLWAYS FORWARD IN SIG GEN MODE
	MOVF      CFGDIR_1,W          ;GET DIRECTION CONFIG INFO
	MOVWF     BAND1
	MOVF      CFGDIR_2,W
	MOVWF     BAND2
	MOVF      BAND,W
          MOVWF     COUNT1              ;BAND CODE NOW IN COUNT1
	BTFSC     COUNT1,3            ;8 OR 9 IF SET
          GOTO      HI_BAND
	INCF      COUNT1,F
ROT       RLF       BAND1,F
	DECFSZ    COUNT1,F
	GOTO      ROT
	BTFSC     CARRY_BIT
	GOTO      SETFORW
	BSF       REVERSE,0           ;REVERSE TUNING
	RETURN
SETFORW   CLRF      REVERSE
	RETURN
HI_BAND
	MOVLW     7
	SUBWF     COUNT1,F            ;8 OR 9 BECOMES 1 OR 2
	MOVF      BAND2,W
	MOVWF     BAND1
	GOTO      ROT

;ROUTINE TO CHECK THE ENTERED BAND HAS BEEN CONFIGURED FOR USE.
;SELECTED BAND IN BAND ON ENTRY, OK = 1 OR 0 ON EXIT.

CHK_BAND
	MOVF      CFGBND_1,W          ;GET ALLOWED BAND INFO
	MOVWF     BAND1
	MOVF      CFGBND_2,W
	MOVWF     BAND2
	MOVF      BAND,W
	MOVWF     COUNT1              ;BAND CODE NOW IN COUNT
	BTFSC     COUNT1,3            ;8 OR 9 IF SET
          GOTO      HI_BD
	INCF      COUNT1,F
ROTATE    RLF       BAND1,F
	DECFSZ    COUNT1,F
	GOTO      ROTATE
	BTFSS     CARRY_BIT
	GOTO      NOGOOD
	MOVLW     1                   ;BAND OK
	MOVWF     OK
	RETURN
NOGOOD    CLRF      OK                  ;BAND NO GOOD
	RETURN
HI_BD
	MOVLW     7
	SUBWF     COUNT1,F            ;8 OR 9 BECOMES 1 OR 2
	MOVF      BAND2,W
	MOVWF     BAND1
	GOTO      ROTATE

;ROUTINE TO READ THE DISPLAY, PAGE 1, AND CONVERT TO BCD IN 4 BYTES
;HWORDH, HWORDL, LWORDH AND LWORDL. SAVES FREQ DOWN TO 10Hz

DISP_2_BCD
	BCF       PCLATH,3
	MOVLW     B'10000000'         ;LCD ADDR 0
	CALL      SND_CMD   
	MOVLW     B'00000110'
	CALL      SND_CMD             ;INCREMENT AFTER EACH READ OR WRITE
	CALL      REC_CHAR            ;GET 10MHz
	MOVWF     TEMP
	BSF       TEMP,4              ;CONVERT BLANK TO 0
	MOVF      TEMP,W
	BSF       PCLATH,3
	CALL      ASCII_BCD
	MOVWF     HWORDH
	SWAPF     HWORDH,F
	BCF       PCLATH,3
	CALL      REC_CHAR            ;GET 1MHz
	BSF       PCLATH,3
	CALL      ASCII_BCD
	IORWF     HWORDH,F         
	BCF       PCLATH,3
	CALL      REC_CHAR            ;GET DOT AND IGNORE         
	CALL      REC_CHAR            ;GET 100KHz
	BSF       PCLATH,3
	CALL      ASCII_BCD
	MOVWF     HWORDL
	SWAPF     HWORDL,F
	BCF       PCLATH,3
	CALL      REC_CHAR            ;GET 10KHz
	BSF       PCLATH,3
	CALL      ASCII_BCD
	IORWF     HWORDL,F       
	BCF       PCLATH,3
	CALL      REC_CHAR            ;GET 1KHz
	BSF       PCLATH,3
	CALL      ASCII_BCD
	MOVWF     LWORDH
	SWAPF     LWORDH,F
	BCF       PCLATH,3
	CALL      REC_CHAR            ;GET 100Hz
	BSF       PCLATH,3
	CALL      ASCII_BCD
	IORWF     LWORDH,F         
	BCF       PCLATH,3
	CALL      REC_CHAR            ;GET 10HZ
	BSF       PCLATH,3
	CALL      ASCII_BCD       
	MOVWF     LWORDL
	SWAPF     LWORDL,F
          MOVF      MODE,W              
          IORWF     LWORDL,F            ;SAVE MODE IN LOW NIBBLE OF LWORDL
	RETURN
ASCII_BCD
	MOVWF     TEMP               
	MOVLW     B'00001111'         ;MASK HIGH BITS
	ANDWF     TEMP,W              ;BCD NOW IN W 
	RETURN

;INITIALISE THE LCD

INIT_LCD
	BCF       PCLATH,3
	CALL      DEBOUNCE            ;20mS DELAY
	CALL      DEBOUNCE
	BCF       RS                  ;INSTRUCTION CODE
          BCF       R_WR                ;DATA WRITE
	MOVLW     B'00111000'         ;8 BIT MODE
	MOVWF     DATA_BUS
	BSF       E  
	BCF       E
	CALL      DEBOUNCE
	MOVLW     B'00111000'         ;8 BIT MODE
	MOVWF     DATA_BUS
	BSF       E  
	BCF       E
	CALL      DEBOUNCE
	MOVLW     B'00111000'         ;8 BIT MODE
	MOVWF     DATA_BUS
	BSF       E  
	BCF       E
	MOVLW     B'00001100'         ;DISPLAY ON, CURSOR OFF, NO BLINK
	CALL      SND_CMD   
	MOVLW     B'00000001'         ;CLEAR DISPLAY, RETURN TO HOME
	CALL      SND_CMD   
	MOVLW     B'00000110'         ;ENTRY MODE, NO DISPLAY SHIFT
	CALL      SND_CMD   
	MOVLW     B'10000000'         ;SET LCD ADDR TO ZERO
	CALL      SND_CMD
	RETURN

;INITIALISE THE DDS

INIT_DDS
	CLRF      DATA_BUS
	BCF       _CS
	BSF       PCLATH,3
	CALL      STROBE              ;LOAD PARALLEL REG WITH 00
	BSF       _CS
	BSF       LOAD                ;TC3 - TC0 STILL = 0000
	BCF       LOAD                ;TRANSFER TO COMMAND REGISTER
	RETURN

;READ CONFIG DATA FROM ADDRESS 204, 205, 206 and 207 OF EEPROM

READ_CONFIG
	MOVLW     D'204'
	MOVWF     ADDR
	BCF       PCLATH,3
	CALL      READ
	BSF       PCLATH,3
	MOVF      HWORDH,W
	MOVWF     CFGBND_1
	MOVF      HWORDL,W
	MOVWF     CFGBND_2
	MOVF      LWORDH,W
	MOVWF     CFGDIR_1
	MOVF      LWORDL,W
	MOVWF     CFGDIR_2
          MOVLW     D'206'
          MOVWF     ADDR
          BCF       PCLATH,3
          CALL      READ
          BSF       PCLATH,3
	MOVF      HWORDH,W
          MOVWF     CFGBFO_1
	MOVF      HWORDL,W
          MOVWF     CFGBFO_2
          MOVF      LWORDH,W
          MOVWF     STEP
          MOVF      LWORDL,W
          MOVWF     SCANDELAY
	RETURN

;READ ADDRESSES 200 AND 201 OF EEPROM, DECODE, DISPLAY AND PROGRAMME
;DDS AND 595'S

READ_00          
	BCF       STATUS,RP0
	MOVLW     D'200'
	MOVWF     ADDR                ;EEPROM ADDRESS 200
	BCF       PCLATH,3
          CALL      READMEM             ;DISPLAY FREQUENCY
          CALL      SET_BFO_DIR         ;BFO OFFSET DIRECTION FOR THE BAND
          BSF       PCLATH,3
          CALL      PR_MODE             ;DISPLAY MODE AND SEND BAND LOGIC
	BSF       PCLATH,3       
          CALL      SET_DIR             ;SET TUNING DIRECTION
          CALL      SND_DDS             ;PROGRAMME DDS
	RETURN


FUNC_1   
	BCF       PCLATH,3
          CALL      CLRPAGE2              ;WIPE PAGE 2
          BTFSC     SIGGEN,0            ;TXRX MODE ONLY
	RETURN
	CALL      SETPAGE2
	MOVLW     "R"
	CALL      SND_CHAR
	MOVLW     "I"
	CALL      SND_CHAR 
	MOVLW     "T"
	CALL      SND_CHAR
          MOVLW     B'11010000'         ;LCD ADDR 50
	CALL      SND_CMD   
	MOVLW     "1"
	CALL      SND_CHAR 
	MOVLW     ":"
	CALL      SND_CHAR
	MOVLW     "O"
	CALL      SND_CHAR 
	MOVLW     "n"
	CALL      SND_CHAR
	MOVLW     " "
	CALL      SND_CHAR 
	MOVLW     "2"
	CALL      SND_CHAR
	MOVLW     ":"
	CALL      SND_CHAR 
	MOVLW     "O"
	CALL      SND_CHAR 
	MOVLW     "f"
	CALL      SND_CHAR
	MOVLW     "f"
	CALL      SND_CHAR
	MOVLW     " "
	CALL      SND_CHAR 
          MOVLW     "3"
	CALL      SND_CHAR
	MOVLW     ":"
	CALL      SND_CHAR 
          MOVLW     "R"
	CALL      SND_CHAR 
          MOVLW     "c"
	CALL      SND_CHAR
          MOVLW     "l"
	CALL      SND_CHAR 
	MOVLW     B'10011111'         ;LCD ADDR 1F
	CALL      SND_CMD   
	MOVF      LASTKEY,W         
	MOVWF     FNCNO               ;FUNCTION NUMBER    
	RETURN
 
FUNC_2   
	BCF       PCLATH,3
          CALL      CLRPAGE2
          BTFSC     SIGGEN,0            ;TXRX MODE ONLY
	RETURN
	CALL      SETPAGE2
	MOVLW     "M"
	CALL      SND_CHAR 
	MOVLW     "o"
	CALL      SND_CHAR
	MOVLW     "d"
	CALL      SND_CHAR 
	MOVLW     "e"
	CALL      SND_CHAR
          MOVLW     " "
          CALL      SND_CHAR
	MOVLW     "1"
	CALL      SND_CHAR
	MOVLW     ":"
	CALL      SND_CHAR 
	MOVLW     "U"
	CALL      SND_CHAR 
	MOVLW     "S"
	CALL      SND_CHAR 
	MOVLW     "B"
	CALL      SND_CHAR
	MOVLW     " "
	CALL      SND_CHAR 
	MOVLW     "2"
	CALL      SND_CHAR
	MOVLW     ":"
	CALL      SND_CHAR 
	MOVLW     "L"
	CALL      SND_CHAR
	MOVLW     "S"
	CALL      SND_CHAR 
	MOVLW     "B"
	CALL      SND_CHAR
	MOVLW     B'11010000'         ;LCD ADDR H50
	CALL      SND_CMD   
	MOVLW     "3"
	CALL      SND_CHAR
	MOVLW     ":"
	CALL      SND_CHAR 
	MOVLW     "C"
	CALL      SND_CHAR
	MOVLW     "W"
	CALL      SND_CHAR
          MOVLW     " "
	CALL      SND_CHAR
          MOVLW     "4"
	CALL      SND_CHAR
	MOVLW     ":"
	CALL      SND_CHAR 
          MOVLW     "A"
	CALL      SND_CHAR
          MOVLW     "M"
	CALL      SND_CHAR
          MOVLW     " "
	CALL      SND_CHAR
          MOVLW     "5"
	CALL      SND_CHAR
	MOVLW     ":"
	CALL      SND_CHAR 
          MOVLW     "F"
	CALL      SND_CHAR
          MOVLW     "M"
	CALL      SND_CHAR 
          MOVLW     B'11011111'         ;LCD ADDR 5F
	CALL      SND_CMD   
	MOVF      LASTKEY,W         
	MOVWF     FNCNO               ;FUNCTION NUMBER    
	RETURN
 
FUNC_3   
	BCF       PCLATH,3
          CALL      CLRPAGE2              ;WIPE PAGE 2
	CALL      SETPAGE2
	CALL      SND_CMD   
	MOVLW     "1"
	CALL      SND_CHAR
	MOVLW     ":"
	CALL      SND_CHAR 
	MOVLW     "1"
	CALL      SND_CHAR
	MOVLW     " "
	CALL      SND_CHAR 
	MOVLW     "2"
	CALL      SND_CHAR
	MOVLW     ":"
	CALL      SND_CHAR 
	MOVLW     "1"
	CALL      SND_CHAR
	MOVLW     "0"
	CALL      SND_CHAR 
	MOVLW     " "
	CALL      SND_CHAR
	MOVLW     "3"
	CALL      SND_CHAR 
	MOVLW     ":"
	CALL      SND_CHAR 
	MOVLW     "1"
	CALL      SND_CHAR
	MOVLW     "0"
	CALL      SND_CHAR 
	MOVLW     "0"
	CALL      SND_CHAR
          MOVLW     B'11010000'         ;LCD ADDR 50
	CALL      SND_CMD   
          MOVLW     "S"
	CALL      SND_CHAR 
          MOVLW     "t"
	CALL      SND_CHAR
          MOVLW     "e"
	CALL      SND_CHAR 
          MOVLW     "p"
	CALL      SND_CHAR 
          MOVLW     "s"
	CALL      SND_CHAR
          MOVLW     " "
	CALL      SND_CHAR 
          MOVLW     "4"
	CALL      SND_CHAR 
          MOVLW     ":"
	CALL      SND_CHAR
          MOVLW     "x"
	CALL      SND_CHAR 
          MOVLW     "2"
	CALL      SND_CHAR 
          MOVLW     " "
	CALL      SND_CHAR
          MOVLW     "5"
	CALL      SND_CHAR 
          MOVLW     ":"
	CALL      SND_CHAR 
          MOVLW     "x"
	CALL      SND_CHAR
          MOVLW     "5"
	CALL      SND_CHAR 
	MOVLW     B'10011111'         ;LCD ADDR 1F
	CALL      SND_CMD   
	MOVF      LASTKEY,W         
	MOVWF     FNCNO               ;FUNCTION NUMBER    
	RETURN

FUNC_4   
	BCF       PCLATH,3
          CALL      CLRPAGE2              ;WIPE PAGE 2
	CALL      SETPAGE2
	MOVLW     "S"
	CALL      SND_CHAR
	MOVLW     "w"
	CALL      SND_CHAR 
	MOVLW     "e"
	CALL      SND_CHAR
	MOVLW     "e"
	CALL      SND_CHAR 
	MOVLW     "p"
	CALL      SND_CHAR
	MOVLW     " "
	CALL      SND_CHAR
	MOVLW     "b"
	CALL      SND_CHAR 
	MOVLW     "a"
	CALL      SND_CHAR
	MOVLW     "n"
	CALL      SND_CHAR 
	MOVLW     "d"
	CALL      SND_CHAR 
	MOVLW     B'11010010'         ;LCD ADDR 52
	CALL      SND_CMD   
	MOVLW     "1"
	CALL      SND_CHAR
	MOVLW     ":"
	CALL      SND_CHAR 
	MOVLW     "U"
	CALL      SND_CHAR
	MOVLW     "p"
	CALL      SND_CHAR 
	MOVLW     " "
	CALL      SND_CHAR
	MOVLW     " "
	CALL      SND_CHAR 
	MOVLW     "2"
	CALL      SND_CHAR
	MOVLW     ":"
	CALL      SND_CHAR 
	MOVLW     "D"
	CALL      SND_CHAR
	MOVLW     "o"
	CALL      SND_CHAR 
	MOVLW     "w"
	CALL      SND_CHAR 
	MOVLW     "n"
	CALL      SND_CHAR
	MOVLW     B'10011111'         ;LCD ADDR 1F
	CALL      SND_CMD   
	MOVF      LASTKEY,W         
	MOVWF     FNCNO               ;FUNCTION NUMBER    
	RETURN
 
FUNC_5   
	BCF       PCLATH,3
          CALL      CLRPAGE2              ;WIPE PAGE 2
          BTFSC     SIGGEN,0            ;TXRX MODE ONLY
	RETURN
	CALL      SETPAGE2
	MOVLW     "S"
	CALL      SND_CHAR
	MOVLW     "c"
	CALL      SND_CHAR 
	MOVLW     "a"
	CALL      SND_CHAR
	MOVLW     "n"
	CALL      SND_CHAR 
	MOVLW     " "
	CALL      SND_CHAR
	MOVLW     "m"
	CALL      SND_CHAR
	MOVLW     "e"
	CALL      SND_CHAR 
	MOVLW     "m"
	CALL      SND_CHAR
	MOVLW     "o"
	CALL      SND_CHAR
	MOVLW     "r"
	CALL      SND_CHAR 
	MOVLW     "i"
	CALL      SND_CHAR
	MOVLW     "e"
	CALL      SND_CHAR 
	MOVLW     "s"
	CALL      SND_CHAR
	MOVLW     B'11010000'         ;LCD ADDR 50
	CALL      SND_CMD   
	MOVLW     "E"
	CALL      SND_CHAR
	MOVLW     "n"
	CALL      SND_CHAR 
	MOVLW     "t"
	CALL      SND_CHAR
	MOVLW     "e"
	CALL      SND_CHAR 
	MOVLW     "r"
	CALL      SND_CHAR
	MOVLW     " "
	CALL      SND_CHAR
	MOVLW     "b"
	CALL      SND_CHAR 
	MOVLW     "a"
	CALL      SND_CHAR
	MOVLW     "n"
	CALL      SND_CHAR 
	MOVLW     "k"
	CALL      SND_CHAR
	MOVLW     " "
	CALL      SND_CHAR 
	MOVLW     "("
	CALL      SND_CHAR 
	MOVLW     "0"
	CALL      SND_CHAR
	MOVLW     "-"
	CALL      SND_CHAR
	MOVLW     "9"
	CALL      SND_CHAR 
	MOVLW     ")"
	CALL      SND_CHAR 
	MOVLW     B'10011111'         ;LCD ADDR 1F
	CALL      SND_CMD   
	MOVF      LASTKEY,W         
	MOVWF     FNCNO               ;FUNCTION NUMBER    
	RETURN

FUNC_6
	BCF       PCLATH,3
          CALL      CLRPAGE2              ;WIPE PAGE 2
	CALL      SETPAGE2
          MOVLW     "O"
	CALL      SND_CHAR 
          MOVLW     "f"
	CALL      SND_CHAR
          MOVLW     "f"
	CALL      SND_CHAR
          MOVLW     "s"
	CALL      SND_CHAR 
          MOVLW     "e"
	CALL      SND_CHAR 
          MOVLW     "t"
	CALL      SND_CHAR
          MOVLW     "s"
	CALL      SND_CHAR
          MOVLW     " "
	CALL      SND_CHAR
	MOVLW     "1"
	CALL      SND_CHAR
	MOVLW     ":"
	CALL      SND_CHAR 
          MOVLW     "O"
	CALL      SND_CHAR
          MOVLW     "n"
	CALL      SND_CHAR 
          MOVLW     B'11010000'         ;LCD ADDR 50
	CALL      SND_CMD   
          MOVLW     "2"
	CALL      SND_CHAR 
          MOVLW     ":"
	CALL      SND_CHAR
          MOVLW     "O"
	CALL      SND_CHAR 
          MOVLW     "f"
	CALL      SND_CHAR
          MOVLW     "f"
	CALL      SND_CHAR
          MOVLW     " "
	CALL      SND_CHAR
          MOVLW     "3"
	CALL      SND_CHAR 
          MOVLW     ":"
	CALL      SND_CHAR
          MOVLW     "S"
	CALL      SND_CHAR 
          MOVLW     "S"
	CALL      SND_CHAR
          MOVLW     "B"
	CALL      SND_CHAR
          MOVLW     " "
	CALL      SND_CHAR
          MOVLW     "o"
	CALL      SND_CHAR 
          MOVLW     "n"
	CALL      SND_CHAR
          MOVLW     "l"
	CALL      SND_CHAR 
          MOVLW     "y"
	CALL      SND_CHAR
	MOVLW     B'10011111'         ;LCD ADDR 1F
	CALL      SND_CMD   
	MOVF      LASTKEY,W         
	MOVWF     FNCNO               ;FUNCTION NUMBER    
	RETURN

FUNC_7
	BCF       PCLATH,3
          CALL      CLRPAGE2              ;WIPE PAGE 2
          BTFSC     SIGGEN,0            ;TXRX MODE ONLY
	RETURN
	CALL      SETPAGE2
	MOVLW     "C"
	CALL      SND_CHAR
	MOVLW     "o"
	CALL      SND_CHAR 
	MOVLW     "n"
	CALL      SND_CHAR
	MOVLW     "f"
	CALL      SND_CHAR 
	MOVLW     "i"
	CALL      SND_CHAR
	MOVLW     "g"
	CALL      SND_CHAR
	MOVLW     " "
	CALL      SND_CHAR 
	MOVLW     "b"
	CALL      SND_CHAR
	MOVLW     "a"
	CALL      SND_CHAR 
	MOVLW     "n"
	CALL      SND_CHAR 
	MOVLW     "d"
	CALL      SND_CHAR 
	MOVLW     "s"
	CALL      SND_CHAR 
	MOVLW     B'11010000'         ;LCD ADDR 50
	CALL      SND_CMD   
	BSF       PCLATH,3
	CALL      READ_CONFIG
	MOVF      CFGBND_1,W
	MOVWF     BAND1
	MOVF      CFGBND_2,W
	MOVWF     BAND2
	CALL      GET_CONFIG
	MOVF      LASTKEY,W   
	MOVWF     FNCNO               ;FUNCTION NUMBER    
	RETURN

FUNC_8   
	BCF       PCLATH,3
          CALL      CLRPAGE2              ;WIPE PAGE 2
          BTFSC     SIGGEN,0            ;TXRX MODE ONLY
	RETURN
	CALL      SETPAGE2
	MOVLW     "C"
	CALL      SND_CHAR
	MOVLW     "o"
	CALL      SND_CHAR 
	MOVLW     "n"
	CALL      SND_CHAR
	MOVLW     "f"
	CALL      SND_CHAR 
	MOVLW     "i"
	CALL      SND_CHAR
	MOVLW     "g"
	CALL      SND_CHAR
	MOVLW     " "
	CALL      SND_CHAR 
	MOVLW     "d"
	CALL      SND_CHAR
	MOVLW     "i"
	CALL      SND_CHAR 
	MOVLW     "r"
	CALL      SND_CHAR 
	MOVLW     "e"
	CALL      SND_CHAR 
	MOVLW     "c"
	CALL      SND_CHAR 
	MOVLW     "t"
	CALL      SND_CHAR
	MOVLW     "i"
	CALL      SND_CHAR 
	MOVLW     "o"
	CALL      SND_CHAR 
	MOVLW     "n"
	CALL      SND_CHAR 
	MOVLW     B'11010000'         ;LCD ADDR 50
	CALL      SND_CMD   
	BSF       PCLATH,3
	CALL      READ_CONFIG
	MOVF      CFGDIR_1,W
	MOVWF     BAND1
	MOVF      CFGDIR_2,W
	MOVWF     BAND2
	CALL      GET_CONFIG
	MOVF      LASTKEY,W   
	MOVWF     FNCNO               ;FUNCTION NUMBER    
	RETURN

FUNC_9
	BCF       PCLATH,3
          CALL      CLRPAGE2              ;WIPE PAGE 2
          BTFSC     SIGGEN,0            ;TXRX MODE ONLY
	RETURN
	CALL      SETPAGE2
	MOVLW     "C"
	CALL      SND_CHAR
	MOVLW     "o"
	CALL      SND_CHAR 
	MOVLW     "n"
	CALL      SND_CHAR
	MOVLW     "f"
	CALL      SND_CHAR 
	MOVLW     "i"
	CALL      SND_CHAR
	MOVLW     "g"
	CALL      SND_CHAR
	MOVLW     " "
	CALL      SND_CHAR 
          MOVLW     "B"
	CALL      SND_CHAR
          MOVLW     "F"
	CALL      SND_CHAR 
          MOVLW     "O"
	CALL      SND_CHAR 
	MOVLW     B'11010000'         ;LCD ADDR 50
	CALL      SND_CMD   
	BSF       PCLATH,3
	CALL      READ_CONFIG
          MOVF      CFGBFO_1,W
	MOVWF     BAND1
          MOVF      CFGBFO_2,W
	MOVWF     BAND2
	CALL      GET_CONFIG
	MOVF      LASTKEY,W   
	MOVWF     FNCNO               ;FUNCTION NUMBER    
	RETURN

GET_CONFIG
	MOVLW     8
	MOVWF     COUNT1
ROTA      RLF       BAND1,F
	BTFSC     CARRY_BIT
	GOTO      PNT_1
	MOVLW     "0"
	BCF       PCLATH,3
	CALL      SND_CHAR
	BSF       PCLATH,3
	GOTO      PNTED
PNT_1     MOVLW     "1"
	BCF       PCLATH,3
	CALL      SND_CHAR
	BSF       PCLATH,3
PNTED     DECFSZ    COUNT1,F
	GOTO      ROTA
	MOVLW     2
	MOVWF     COUNT1
RO        RLF       BAND2,F
	BTFSC     CARRY_BIT
	GOTO      PN_1
	MOVLW     "0"
	BCF       PCLATH,3
	CALL      SND_CHAR
	BSF       PCLATH,3
	GOTO      PTED
PN_1      MOVLW     "1"
	BCF       PCLATH,3
	CALL      SND_CHAR
	BSF       PCLATH,3
PTED      DECFSZ    COUNT1,F
	GOTO      RO
	MOVLW     B'11010000'         ;LCD ADDR 50
	BCF       PCLATH,3
	CALL      SND_CMD   
          MOVLW     D'10'
	MOVWF     KEYCOUNT
	RETURN

FUNC_DOT
	BCF       PCLATH,3
          CALL      CLRPAGE2              ;WIPE PAGE 2
	CALL      SETPAGE2
	CALL      SND_CMD
          MOVLW     "B"
	CALL      SND_CHAR
          MOVLW     "e"
	CALL      SND_CHAR
          MOVLW     "e"
	CALL      SND_CHAR 
          MOVLW     "p"
	CALL      SND_CHAR 
	MOVLW     B'11010011'         ;LCD ADDR 53
	CALL      SND_CMD   
	MOVLW     "1"
	CALL      SND_CHAR 
	MOVLW     ":"
	CALL      SND_CHAR
	MOVLW     "O"
	CALL      SND_CHAR 
	MOVLW     "n"
	CALL      SND_CHAR
	MOVLW     " "
	CALL      SND_CHAR 
	MOVLW     "2"
	CALL      SND_CHAR
	MOVLW     ":"
	CALL      SND_CHAR 
	MOVLW     "O"
	CALL      SND_CHAR 
	MOVLW     "f"
	CALL      SND_CHAR
	MOVLW     "f"
	CALL      SND_CHAR 
	MOVLW     B'10011111'         ;LCD ADDR 1F
	CALL      SND_CMD   
	MOVF      LASTKEY,W         
	MOVWF     FNCNO               ;FUNCTION NUMBER    
	RETURN

FUNC_STO
	BCF       PCLATH,3
          CALL      CLRPAGE2              ;WIPE PAGE 2
	CALL      SETPAGE2
	MOVLW     "O"
	CALL      SND_CHAR
	MOVLW     "u"
	CALL      SND_CHAR 
	MOVLW     "t"
	CALL      SND_CHAR 
	MOVLW     "p"
	CALL      SND_CHAR
	MOVLW     "u"
	CALL      SND_CHAR 
	MOVLW     "t"
	CALL      SND_CHAR
	MOVLW     " "
	CALL      SND_CHAR
	MOVLW     "("
	CALL      SND_CHAR 
	MOVLW     "M"
	CALL      SND_CHAR 
	MOVLW     "H"
	CALL      SND_CHAR
	MOVLW     "z"
	CALL      SND_CHAR 
	MOVLW     ")"
	CALL      SND_CHAR
	MOVLW     B'11010001'         ;LCD ADDR 51
	CALL      SND_CMD   
	MOVLW     "1"
	CALL      SND_CHAR
	MOVLW     ":"
	CALL      SND_CHAR 
	MOVLW     "5"
	CALL      SND_CHAR
	MOVLW     "-"
	CALL      SND_CHAR 
	MOVLW     "6"
	CALL      SND_CHAR
	MOVLW     " "
	CALL      SND_CHAR 
	MOVLW     " "
	CALL      SND_CHAR
	MOVLW     "2"
	CALL      SND_CHAR 
	MOVLW     ":"
	CALL      SND_CHAR
	MOVLW     "D"
	CALL      SND_CHAR 
	MOVLW     "C"
	CALL      SND_CHAR 
	MOVLW     "-"
	CALL      SND_CHAR
	MOVLW     "1"
	CALL      SND_CHAR 
	MOVLW     "5"
	CALL      SND_CHAR  
	MOVLW     B'10011111'         ;LCD ADDR 1F
	CALL      SND_CMD   
	MOVF      LASTKEY,W         
	MOVWF     FNCNO               ;FUNCTION NUMBER    
	RETURN


FUNC_RCL   
	BCF       PCLATH,3
          CALL      CLRPAGE2              ;WIPE PAGE 2
          BTFSS     SIGGEN,0            ;SIG GEN MODE ONLY
          GOTO      FUNC_RCL2
	CALL      SETPAGE2
	MOVLW     B'00001100'         ;CURSOR OFF NO BLINK
	CALL      SND_CMD
	MOVLW     " "
	CALL      SND_CHAR
	MOVLW     "S"
	CALL      SND_CHAR
	MOVLW     "o"
	CALL      SND_CHAR 
	MOVLW     "f"
	CALL      SND_CHAR 
	MOVLW     "t"
	CALL      SND_CHAR
	MOVLW     "w"
	CALL      SND_CHAR 
	MOVLW     "a"
	CALL      SND_CHAR
	MOVLW     "r"
	CALL      SND_CHAR
	MOVLW     "e"
	CALL      SND_CHAR 
	MOVLW     " "
	CALL      SND_CHAR 
	MOVLW     "V"
	CALL      SND_CHAR
	MOVLW     "1"
	CALL      SND_CHAR 
	MOVLW     "."
	CALL      SND_CHAR
          MOVLW     "9"
	CALL      SND_CHAR
          MOVLW     "4"
	CALL      SND_CHAR
	MOVLW     H'5F'
	MOVWF     WRADDR
	MOVLW     "G"
	MOVWF     CS_G
	MOVLW     "4"
	MOVWF     CS_4
	MOVLW     "O"
	MOVWF     CS_O
	MOVLW     "P"
	MOVWF     CS_P
	MOVLW     "E"
	MOVWF     CS_E
	MOVLW     " "
	MOVWF     CS_
	MOVLW     D'16'
	MOVWF     COUNT1
NEXTOF16   
	MOVLW     CS_G
	MOVWF     FSR
	BSF       WRADDR,7
	MOVF      WRADDR,W
	BCF       PCLATH,3
	CALL      SND_CMD
	BSF       PCLATH,3
	BCF       WRADDR,7
	MOVLW     6
	MOVWF     COUNT
NEXTOF6
	MOVF      INDF,W
	BCF       PCLATH,3
	CALL      SND_CHAR
	BSF       PCLATH,3  
	INCF      FSR,F
	DECFSZ    COUNT,F
	GOTO      NEXTOF6
	BCF       PCLATH,3
	CALL      DEBOUNCE
	CALL      DEBOUNCE
	CALL      DEBOUNCE
	BSF       PCLATH,3
	DECF      WRADDR,F     
	DECFSZ    COUNT1,F
	GOTO      NEXTOF16
	BCF       PCLATH,3
DEL_G4OPE
	CALL      PAUSE
	CALL      PAUSE
	CALL      PAUSE
	CALL      PAUSE
	CALL      PAUSE
	MOVLW     B'11010000'         ;LCD ADDR 50
	CALL      SND_CMD
	MOVLW     " "
	CALL      SND_CHAR
	CALL      SND_CHAR
	CALL      SND_CHAR
	CALL      SND_CHAR
	CALL      SND_CHAR
	MOVF      LASTKEY,W         
	MOVWF     FNCNO               ;FUNCTION NUMBER    
	RETURN

BEEP
          BCF       STATUS,RP0
          CLRF      COUNT1
CYCLE
          MOVLW     D'40'
          MOVWF     COUNT
          BCF       SPEAKER
OUTLOW
          DECFSZ    COUNT,F
          GOTO      OUTLOW
          MOVLW     D'40'
          MOVWF     COUNT
          BSF       SPEAKER
OUTHIGH
          DECFSZ    COUNT,F
          GOTO      OUTHIGH
          DECFSZ    COUNT1,F
          GOTO      CYCLE
          RETURN

;FUNCTION 1 SELECTED AND CHOICE ENTERED. ROUTINE TO SERVICE THE CHOICE MADE.

FUNC1
	BCF       STATUS,RP0
	MOVF      LASTKEY,W
          BCF       PCLATH,3
	CALL      BCD_ASCII
	CALL      SND_CHAR            ;PRINT CHOICE
	CALL      PAUSE
          BSF       PCLATH,3
	MOVF      LASTKEY,W
	MOVLW     1
	SUBWF     LASTKEY,W
	BTFSC     ZERO_BIT
	GOTO      SEL_RIT             ;1 ENTERED
          MOVLW     2
	SUBWF     LASTKEY,W
	BTFSC     ZERO_BIT
          GOTO      DESEL_RIT           ;2 ENTERED
          MOVLW     3
	SUBWF     LASTKEY,W
	BTFSC     ZERO_BIT
          GOTO      RCL_RIT             ;3 ENTERED
          RETURN
RCL_RIT
          BSF       RIT,0            
          CALL      RECALL_RIT
          RETURN
DESEL_RIT                               
	BCF       RIT,0
	CALL      DEL_RIT
	RETURN
SEL_RIT
	BSF       RIT,0
	CALL      PNT_RIT
	RETURN
PNT_RIT
	MOVLW     B'11000100'         ;LCD ADDR 44
          BCF       PCLATH,3
	CALL      SND_CMD
	MOVLW     " "
	CALL      SND_CHAR
	MOVLW     "0"
	CALL      SND_CHAR
	CALL      SND_CHAR
	CALL      SND_CHAR
	CALL      SND_CHAR
	RETURN
DEL_RIT
	MOVLW     B'11000100'         ;LCD ADDR 44
          BCF       PCLATH,3
	CALL      SND_CMD
	MOVLW     5
	MOVWF     COUNT
	MOVLW     " "
SD_SPACE  BCF       PCLATH,3
          CALL      SND_CHAR
          BSF       PCLATH,3
	DECFSZ    COUNT,F
	GOTO      SD_SPACE
	RETURN

RECALL_RIT
	MOVLW     B'11000100'         ;LCD ADDR 44
          BCF       PCLATH,3
	CALL      SND_CMD
          MOVLW     RIT_SIGN
          MOVWF     FSR
	MOVLW     5
	MOVWF     COUNT
GET_RIT   BCF       PCLATH,3
          MOVF      INDF,W
          CALL      SND_CHAR
          BSF       PCLATH,3
          INCF      FSR,F
          DECFSZ    COUNT,F
          GOTO      GET_RIT
          RETURN
          
FUNC2     
	BCF       STATUS,RP0
	MOVF      LASTKEY,W
          BCF       PCLATH,3
	CALL      BCD_ASCII
	CALL      SND_CHAR            ;PRINT CHOICE
	CALL      PAUSE
          BSF       PCLATH,3
	MOVF      LASTKEY,W
	MOVLW     1
	SUBWF     LASTKEY,W
	BTFSC     ZERO_BIT            ;1?
	GOTO      _USB
	MOVLW     2
	SUBWF     LASTKEY,W
	BTFSC     ZERO_BIT            ;2?
	GOTO      _LSB
	MOVLW     3
	SUBWF     LASTKEY,W
	BTFSC     ZERO_BIT            ;3?
	GOTO      _CW
          MOVLW     4
	SUBWF     LASTKEY,W
          BTFSC     ZERO_BIT            ;4?
          GOTO      _AM
          MOVLW     5
	SUBWF     LASTKEY,W
          BTFSC     ZERO_BIT            ;5?
          GOTO      _FM
          BCF       PCLATH,3
          RETURN
_FM       MOVLW     1
          MOVWF     MODE
          GOTO      DISP_MODE
_AM       MOVLW     2
          MOVWF     MODE
          GOTO      DISP_MODE
_CW       MOVLW     3
          MOVWF     MODE
          GOTO      DISP_MODE
_LSB      MOVLW     4
          MOVWF     MODE
          GOTO      DISP_MODE
_USB      MOVLW     5
          MOVWF     MODE
DISP_MODE
          CALL      DESEL_RIT
          CALL      PR_MODE
          CALL      SND_DDS
          RETURN
PR_MODE
	MOVLW     B'11001010'         ;LCD ADDR 4A
          BCF       PCLATH,3
	CALL      SND_CMD
          BSF       PCLATH,3
          MOVF      MODE,F
	BTFSC     ZERO_BIT
	GOTO      NO_MODE
          MOVLW     5
          SUBWF     MODE,W
          BTFSC     ZERO_BIT
          GOTO      PR_USB
          MOVLW     4
          SUBWF     MODE,W
          BTFSC     ZERO_BIT
          GOTO      PR_LSB
          MOVLW     3
          SUBWF     MODE,W
          BTFSC     ZERO_BIT
          GOTO      PR_CW
          MOVLW     2
          SUBWF     MODE,W
          BTFSC     ZERO_BIT
          GOTO      PR_AM
PR_FM     BCF       PCLATH,3
          MOVLW     " "                 ;MUST BE FM
	CALL      SND_CHAR
          MOVLW     "F"
	CALL      SND_CHAR
          MOVLW     "M"
	CALL      SND_CHAR
          BSF       PCLATH,3
	GOTO      PR
PR_AM     BCF       PCLATH,3
          MOVLW     " "
	CALL      SND_CHAR
          MOVLW     "A"
	CALL      SND_CHAR
          MOVLW     "M"
	CALL      SND_CHAR
          BSF       PCLATH,3
	GOTO      PR
PR_CW     BCF       PCLATH,3
          MOVLW     " "
	CALL      SND_CHAR
	MOVLW     "C"
	CALL      SND_CHAR
	MOVLW     "W"
	CALL      SND_CHAR
          BSF       PCLATH,3
	GOTO      PR
PR_USB    BCF       PCLATH,3
          MOVLW     "U"
	CALL      SND_CHAR
          BSF       PCLATH,3
	GOTO      PR_SB
PR_LSB    BCF       PCLATH,3
          MOVLW     "L"
	CALL      SND_CHAR
PR_SB     BCF       PCLATH,3
          MOVLW     "S"
	CALL      SND_CHAR
	MOVLW     "B"
	CALL      SND_CHAR
          BSF       PCLATH,3
	GOTO      PR
NO_MODE   BCF       PCLATH,3
          MOVLW     " "
	CALL      SND_CHAR
	CALL      SND_CHAR
	CALL      SND_CHAR
PR        BSF       PCLATH,3
	CALL      SND_BAND
	BTFSS     SIGGEN,0
	GOTO      S200
	MOVLW     D'202'
	GOTO      SA
S200      MOVLW     D'200'
SA        MOVWF     ADDR
          BCF       PCLATH,3
	CALL      STOREDISP
          BCF       PCLATH,3
          CALL      SET_BFO_DIR
          BSF       PCLATH,3
	RETURN

FUNC3     
	BCF       STATUS,RP0
	MOVF      LASTKEY,W
          BCF       PCLATH,3
	CALL      BCD_ASCII
	CALL      SND_CHAR            ;PRINT CHOICE
	CALL      PAUSE
          BSF       PCLATH,3
	MOVF      LASTKEY,W
	BTFSC     ZERO_BIT            ;CHECK FOR 0 ENTERED
          RETURN
          MOVLW     6
	BCF       CARRY_BIT
	SUBWF     LASTKEY,W         
          BTFSC     CARRY_BIT           ;CHECK FOR GREATER THAN 5 ENTERED
          RETURN
	MOVLW     1
	SUBWF     LASTKEY,W
          BTFSS     ZERO_BIT
          GOTO      TRY2
          CLRF      STEP
	BSF       STEP,0
          GOTO      TESTRIT
TRY2      MOVLW     2
	SUBWF     LASTKEY,W
          BTFSS     ZERO_BIT
          GOTO      TRY3
          CLRF      STEP
	BSF       STEP,1
          GOTO      TESTRIT
TRY3      MOVLW     3
	SUBWF     LASTKEY,W
          BTFSS     ZERO_BIT
          GOTO      TRY4
          CLRF      STEP
          BSF       STEP,2
          GOTO      TESTRIT
TRY4      MOVLW     4
	SUBWF     LASTKEY,W
          BTFSS     ZERO_BIT
          GOTO      TRY5
          BCF       STEP,4
          BSF       STEP,3
          GOTO      TESTRIT
TRY5      BCF       STEP,3
          BSF       STEP,4              ;MUST BE A 5
TESTRIT   BCF       PCLATH,3
          CALL      STO_CONFIG
          BSF       PCLATH,3
          BTFSC     RIT,0
	GOTO      ZE_TAIL
TAILOK    RETURN
ZE_TAIL   BTFSC     STEP,0
          RETURN
	BTFSC     STEP,1
	GOTO      ZE_LAST
	MOVLW     B'11000111'         ;LCD ADDR 47
          BCF       PCLATH,3
	CALL      SND_CMD
	MOVLW     "0"
	CALL      SND_CHAR
ZE_LAST   MOVLW     B'11001000'         ;LCD ADDR 48
          BCF       PCLATH,3
	CALL      SND_CMD
	MOVLW     "0"
	CALL      SND_CHAR
	BSF       PCLATH,3
	CALL      SND_DDS
          MOVF      _1KHz,W             ;CHECK FOR RIT = 0000
          BTFSS     ZERO_BIT
          RETURN
          MOVF      _100Hz,W
          BTFSS     ZERO_BIT
          RETURN
          MOVF      _10Hz,W
          BTFSS     ZERO_BIT
          RETURN
          MOVF      _1Hz,W
          BTFSS     ZERO_BIT
          RETURN
          MOVLW     B'11000100'         ;LCD ADDR 44
          BCF       PCLATH,3
          CALL      SND_CMD
          MOVLW     " "
          CALL      SND_CHAR           ;DELETE SIGN IF RIT = 0000
          RETURN

FUNC4
	BCF       STATUS,RP0
	MOVF      LASTKEY,W
          BCF       PCLATH,3
	CALL      BCD_ASCII
	CALL      SND_CHAR            ;PRINT CHOICE
	CALL      PAUSE
          BSF       PCLATH,3
	MOVF      LASTKEY,W
	BTFSC     ZERO_BIT            ;CHECK FOR 0 ENTERED
          RETURN
	MOVLW     3
	BCF       CARRY_BIT
	SUBWF     LASTKEY,W         
	BTFSC     CARRY_BIT           ;CHECK FOR GREATER THAN 2 ENTERED
          RETURN
	MOVLW     B'10001110'         ;LCD ADDR 0E
          BCF       PCLATH,3
	CALL      SND_CMD
	MOVLW     " "                 ;CLEAR ANY MEMORY LOCATION
	CALL      SND_CHAR
	CALL      SND_CHAR
          BSF       PCLATH,3
	MOVLW     1
	SUBWF     LASTKEY,W
	BTFSC     ZERO_BIT
	GOTO      SW_UP               ;1 ENTERED
SW_DWN    BSF       DIRECN,0
	GOTO      SET_SW
SW_UP     BCF       DIRECN,0
SET_SW    BSF       SWEEP,0
	BTFSC     RIT,0
	CALL      DESEL_RIT
	RETURN

FUNC5     
	BCF       STATUS,RP0
	MOVF      LASTKEY,W
          BCF       PCLATH,3
	CALL      BCD_ASCII
	CALL      SND_CHAR            ;PRINT CHOICE
	CALL      PAUSE
          BSF       PCLATH,3
	MOVLW     D'10'
	BCF       CARRY_BIT
	SUBWF     LASTKEY,W         
	BTFSC     CARRY_BIT           ;CHECK FOR GREATER THAN 9 ENTERED
          RETURN
	CLRF      MEMLOW              ;LOW DIGIT OF MEM LOCATION STARTS
	MOVF      LASTKEY,W           ;OFF AS ZERO.
	MOVWF     MEMHIGH             ;HIGH DIGIT OF MEM LOCATION
          BCF       PCLATH,3
	CALL      MULT_X_10
          BSF       PCLATH,3
	MOVWF     ADDR
	BCF       CARRY_BIT
	RLF       ADDR,F
	BSF       SCAN,0
	MOVLW     D'10'
          MOVWF     SCANCOUNT           ;KEEPS COUNT OF THE MEMORIES SCANNED
          CLRF      USEDCOUNT           ;COUNTS NO. OF USED MEMORIES IN BANK
	BTFSC     RIT,0
	CALL      DESEL_RIT
	RETURN

FUNC6
	BCF       STATUS,RP0
	MOVF      LASTKEY,W
          BCF       PCLATH,3
	CALL      BCD_ASCII
	CALL      SND_CHAR            ;PRINT CHOICE
	CALL      PAUSE
          BSF       PCLATH,3
	MOVF      LASTKEY,W
	MOVLW     1
	SUBWF     LASTKEY,W
	BTFSC     ZERO_BIT
          GOTO      ENABLEBOTH
          MOVLW     2
          SUBWF     LASTKEY,W
          BTFSC     ZERO_BIT
          GOTO      NOBFO
          MOVLW     3
	SUBWF     LASTKEY,W
          BTFSS     ZERO_BIT
          RETURN
SSBONLY
          BSF       CFGBFO_2,0          ;XXXXXX10 = SSB ONLY
          BCF       CFGBFO_2,1
          CALL      SAVEBFO
          RETURN
NOBFO
          BCF       CFGBFO_2,0          ;XXXXXX00 = NO OFFSETS
          BCF       CFGBFO_2,1
          CALL      SAVEBFO
          RETURN
ENABLEBOTH              
          BSF       CFGBFO_2,0          ;XXXXXX11 = BOTH SSB AND CW
          BSF       CFGBFO_2,1
          CALL      SAVEBFO
          RETURN
SAVEBFO
          BCF       PCLATH,3
          CALL      STO_CONFIG          ;SAVE BFO ON/OFF SETTING
          BCF       PCLATH,3
          CALL      SET_BFO_DIR
          BSF       PCLATH,3
          CALL      SND_DDS
	RETURN

FUNC7
	BCF       STATUS,RP0
	MOVLW     D'12'               ;ENTER KEY
	SUBWF     LASTKEY,W
	BTFSC     ZERO_BIT
          GOTO      WRT_BND_CFG
          MOVLW     D'11'               ;CLEAR KEY
	SUBWF     LASTKEY,W
	BTFSC     ZERO_BIT
          GOTO      ABANDON
	MOVLW     2                   ;CHECK LESS THAN 2 ENTERED
	SUBWF     LASTKEY,W
	BTFSC     CARRY_BIT
          RETURN
          MOVF      LASTKEY,W
          BCF       PCLATH,3
	CALL      BCD_ASCII
	CALL      SND_CHAR
          DECFSZ    KEYCOUNT,F          ;CHECK TEN DIGITS ENTERED
          RETURN
WRT_BND_CFG
	MOVLW     B'11010000'         ;LCD ADDR 50
          BCF       PCLATH,3
	CALL      SND_CMD
	MOVLW     8
	MOVWF     COUNT1
NXT_CH    BCF       PCLATH,3
          CALL      REC_CHAR
	BSF       PCLATH,3
	CALL      ASCII_BCD
	MOVWF     TEMP                ;MUST BE 1 OR 0
	RRF       TEMP,F
	RLF       CFGBND_1,F
	DECFSZ    COUNT1,F
	GOTO      NXT_CH
	MOVLW     2
	MOVWF     COUNT1
NXT_C     BCF       PCLATH,3
          CALL      REC_CHAR
	BSF       PCLATH,3
	CALL      ASCII_BCD
	MOVWF     TEMP                ;MUST BE 1 OR 0
	RRF       TEMP,F
	RLF       CFGBND_2,F
	DECFSZ    COUNT1,F
	GOTO      NXT_C
	MOVLW     6
	MOVWF     COUNT1
ROT6      RLF       CFGBND_2,F
	DECFSZ    COUNT1,F
	GOTO      ROT6
          BCF       PCLATH,3
	CALL      STO_CONFIG
          CALL      CLRPAGE2
	RETURN

FUNC8
	BCF       STATUS,RP0
	MOVLW     D'12'               ;ENTER KEY
	SUBWF     LASTKEY,W
	BTFSC     ZERO_BIT
          GOTO      WRT_DIR_CFG
          MOVLW     D'11'               ;CLEAR KEY
	SUBWF     LASTKEY,W
	BTFSC     ZERO_BIT
          GOTO      ABANDON
	MOVLW     2                   ;CHECK LESS THAN 2 ENTERED
	SUBWF     LASTKEY,W
	BTFSC     CARRY_BIT
          RETURN
          MOVF      LASTKEY,W
          BCF       PCLATH,3
	CALL      BCD_ASCII
	CALL      SND_CHAR
          DECFSZ    KEYCOUNT,F          ;CHECK TEN DIGITS ENTERED
          RETURN
WRT_DIR_CFG
	MOVLW     B'11010000'         ;LCD ADDR 50
          BCF       PCLATH,3
	CALL      SND_CMD
	MOVLW     8
	MOVWF     COUNT1
NEXT_CH   BCF       PCLATH,3
          CALL      REC_CHAR
	BSF       PCLATH,3
	CALL      ASCII_BCD
	MOVWF     TEMP                ;MUST BE 1 OR 0
	RRF       TEMP,F
	RLF       CFGDIR_1,F
	DECFSZ    COUNT1,F
	GOTO      NEXT_CH
	MOVLW     2
	MOVWF     COUNT1
NEXT_C    BCF       PCLATH,3
          CALL      REC_CHAR
	BSF       PCLATH,3
	CALL      ASCII_BCD
	MOVWF     TEMP                ;MUST BE 1 OR 0
	RRF       TEMP,F
	RLF       CFGDIR_2,F
	DECFSZ    COUNT1,F
	GOTO      NEXT_C
	MOVLW     6
	MOVWF     COUNT1
RO6       RLF       CFGDIR_2,F
	DECFSZ    COUNT1,F
	GOTO      RO6
          BCF       PCLATH,3
	CALL      STO_CONFIG
          BSF       PCLATH,3
          CALL      SET_DIR
          CALL      SND_DDS
          BCF       PCLATH,3
          CALL      CLRPAGE2
	RETURN

FUNC9
	BCF       STATUS,RP0
	MOVLW     D'12'               ;ENTER KEY
	SUBWF     LASTKEY,W
	BTFSC     ZERO_BIT
          GOTO      WRT_BFO_CFG
          MOVLW     D'11'               ;CLEAR KEY
	SUBWF     LASTKEY,W
	BTFSC     ZERO_BIT
          GOTO      ABANDON
	MOVLW     2                   ;CHECK LESS THAN 2 ENTERED
	SUBWF     LASTKEY,W
	BTFSC     CARRY_BIT
          RETURN
          MOVF      LASTKEY,W
          BCF       PCLATH,3
	CALL      BCD_ASCII
	CALL      SND_CHAR
          DECFSZ    KEYCOUNT,F          ;CHECK TEN DIGITS ENTERED
	RETURN
WRT_BFO_CFG
	MOVLW     B'11010000'         ;LCD ADDR 50
          BCF       PCLATH,3
	CALL      SND_CMD
	MOVLW     8
	MOVWF     COUNT1
NXT_CHR   BCF       PCLATH,3
          CALL      REC_CHAR
	BSF       PCLATH,3
	CALL      ASCII_BCD
	MOVWF     TEMP                ;MUST BE 1 OR 0
	RRF       TEMP,F
          RLF       CFGBFO_1,F
	DECFSZ    COUNT1,F
          GOTO      NXT_CHR
	MOVLW     2
	MOVWF     COUNT1
NXT_CHAR  BCF       PCLATH,3
          CALL      REC_CHAR
	BSF       PCLATH,3
	CALL      ASCII_BCD
	MOVWF     TEMP                ;MUST BE 1 OR 0
	RRF       TEMP,F
          RLF       CFGBFO_2,F
	DECFSZ    COUNT1,F
          GOTO      NXT_CHAR
	MOVLW     6
	MOVWF     COUNT1
          CLRF      TEMP
ROTATE6   BCF       CARRY_BIT
          RLF       CFGBFO_2,F
          RLF       TEMP,F
	DECFSZ    COUNT1,F
          GOTO      ROTATE6
          MOVF      TEMP,W
          IORWF     CFGBFO_2,F
          BCF       PCLATH,3
	CALL      STO_CONFIG
          BSF       PCLATH,3
          CALL      PR_MODE             ;DISPLAY MODE AND SEND BAND LOGIC
	BSF       PCLATH,3       
          CALL      SET_DIR             ;SET TUNING DIRECTION
          CALL      SND_DDS             ;PROGRAMME DDS
          BCF       PCLATH,3
          CALL      CLRPAGE2
	RETURN
ABANDON
          BCF       PCLATH,3
          GOTO      CLRALL

FUNCDOT
	BCF       STATUS,RP0
	MOVF      LASTKEY,W
          BCF       PCLATH,3
	CALL      BCD_ASCII
	CALL      SND_CHAR            ;PRINT CHOICE
	CALL      PAUSE
          BSF       PCLATH,3
	MOVF      LASTKEY,W
	MOVLW     1
	SUBWF     LASTKEY,W
          BTFSC     ZERO_BIT
          GOTO      SETBEEP
          MOVLW     2
	SUBWF     LASTKEY,W
          BTFSS     ZERO_BIT
          RETURN
          BSF       CFGBFO_2,2          ;2 ENTERED
          BCF       PCLATH,3
          CALL      STO_CONFIG
          RETURN
SETBEEP   BCF       CFGBFO_2,2          ;1 ENTERED
          BCF       PCLATH,3
          CALL      STO_CONFIG
          RETURN

FUNCSTO  
	BCF       STATUS,RP0
	MOVF      LASTKEY,W
          BCF       PCLATH,3
	CALL      BCD_ASCII
	CALL      SND_CHAR            ;PRINT CHOICE
	CALL      PAUSE
          BSF       PCLATH,3
	MOVF      LASTKEY,W
	BTFSC     ZERO_BIT            ;CHECK FOR 0 ENTERED
          RETURN
	MOVLW     3
	BCF       CARRY_BIT
	SUBWF     LASTKEY,W         
	BTFSC     CARRY_BIT           ;CHECK FOR GREATER THAN 2 ENTERED
          RETURN
	MOVLW     H'FF'               ;STOPS LAST MEMORY LOCATION
	MOVWF     MEMLOW              ;RECALLED FROM BEING PRINTED
          BSF       PCLATH,3
	MOVLW     1
	SUBWF     LASTKEY,W
	BTFSC     ZERO_BIT
	GOTO      SEL_TXRX            ;1 ENTERED
SEL_SGEN                              
	BSF       SIGGEN,0
	BCF       RA0
	CLRF      MODE  
	MOVLW     D'202'
	MOVWF     ADDR
          BCF       PCLATH,3
	CALL      READMEM
          BSF       PCLATH,3
	CALL      PR_MODE
	BSF       PCLATH,3
	CALL      SND_DDS
	RETURN
SEL_TXRX
	CLRF      SIGGEN
	BSF       RA0
	MOVLW     D'200'
	MOVWF     ADDR
          BCF       PCLATH,3
	CALL      READMEM
          CALL      SET_BFO_DIR
          BSF       PCLATH,3
	CALL      PR_MODE
	BSF       PCLATH,3
          CALL      SET_DIR
	CALL      SND_DDS
	RETURN


FUNCRCL   BTFSC     SIGGEN,0
          RETURN
          BCF       PCLATH,3
          GOTO      FUNCRCL2

	END
