;         PIC16C54 IAMBIC KEYER WITH SINGLE MEMORY.
;
;         Mick Hodges         mick@g4ope.demon.co.uk
;
;The circuit for this keyer appeared in SPRAT issue number 86, Spring 1996.
;
;Clock runs between 8khz and 25khz for a speed range of about 12wpm to 38wpm.
;Use a 330pF capacitor with a 47k pot and 27k fixed resistor.
;
;Output is taken from RB6 or RB7.
;A short message can be recorded by grounding RA3 and will remain in memory
;until power is removed. RB5 is used to light a LED while recording.
;The stored message, correctly spaced, is sent by momentarily grounding RA2.
;RB4 is used to light a LED when sending the stored message.
;Hitting the paddle aborts sending of the stored message.
;
;Keyer logic.
;The status of the paddle is sampled 10 times during each dot period.
;While sending a dot, or the space following the dot, a hit on the dash 
;paddle is stored in memory but a hit on the dot paddle is ignored.
;While sending a dash, or the space following the dash, a hit on the dot 
;paddle is stored in memory but a hit on the dash paddle is ignored.
;Any dot or dash, captured while sending a dot or dash, is sent after first 
;completing the space.
;When sending a consecutive space, a hit on either paddle will abort the 
;space and the dot or dash will be sent immediately.
;
;
;
;
;         Connect as follows
;
;         RA0       Dot
;         RA1       Dash
;         RA2       Send message switch
;         RA3       Record message switch
;         RB0-RB3   Weight control
;                   Wiring of pins 0 to 3 (16 settings)
;                   3210
;                   1000 for equal dot/space ratio
;                   0000 for short spaces (approx. half dot lenght)
;                   1111 for long spaces (approx. 1.5 dot lengths)
;         RB4       High when sending message
;         RB5       High when recording
;         RB6       Keying LED
;         RB7       Keying transistor
;
          list      p=16c54
          include   <p16c5x.inc>
          include   <p16c5xd.inc>
;
          flags     equ       01        ;(TMR0 not used as a timer)
                                        ;bit 0 dot memory
                                        ;bit 1 dash memory
                                        ;bit 2 dot sent last
                                        ;bit 3 dash sent last
                                        ;bit 4 space sent last
                                        ;bit 5 end of message when sending
                                        ;bit 6 message being sent
                                        ;bit 7 recording message
          inner     equ       07
          outer     equ       08

          count     equ       09
          fourchrs  equ       0A
          chr       equ       0B
          messag    equ       0C
          spcecnt   equ       1F
;
init
          movlw     b'00001111'         ;0 - 3 input, 4 - 7 output          
          tris      PORTB
          movlw     0x00
          movwf     PORTB               ;all outputs low
          movlw     messag
          movwf     FSR                 ;message pointer
          clrf      flags
          movlw     0x04
          movwf     count
          movlw     0xFF    
          movwf     messag
          movwf     spcecnt
          goto      send_space
send_message
          btfss     RA3                 ;recording?
          goto      send_space          ;yes so wait till button lifted
          bsf       flags,6             ;no so ok to send message
          bsf       RB4                 ;used to light LED
next_four
          movlw     0x04                ;4 chrs per byte
          movwf     count
          movf      INDF,w
          movwf     fourchrs            ;next 4 chrs
next_chr
          clrf      chr
          rlf       fourchrs,f
          rlf       chr,f
          rlf       fourchrs,f
          rlf       chr,f               ;next chr rotated in
          movf      chr,w               ;moved into w
          addwf     PCL,f               ;what is it?
          goto      dot                 ;00 = dot
          goto      dash                ;01 = dash
          goto      two_spaces          ;10 = end of letter
          goto      six_spaces          ;11 = end of word
one_space
          movlw     d'10'
          movwf     outer
          bcf       flags,5             ;clear end of word flag
          goto      _space
two_spaces
          movlw     d'20'
          movwf     outer
          bcf       flags,5             ;clear end of word flag
          goto      _space
six_spaces
          btfsc     flags,5             ;first end of word chr?
          goto      abort               ;no so end of message
          movlw     d'60'     
          movwf     outer
          bsf       flags,5             ;flag first end of word
_space
          bcf       RB6
          bcf       RB7                 ;output a low
          movf      PORTA,w
          andlw     b'00000011'         ;see if the paddle has been hit
          addwf     PCL,f
          goto      abort               ;00 both
          goto      abort               ;01 dash
          goto      abort               ;10 dot
outer_loop
          movlw     d'15'               ;11 neither
          movwf     inner
          movf      PORTB,w 
          andlw     b'00001111'       
          addwf     inner,f             ;adjust weight 
inner_loop
          decfsz    inner,f
          goto      inner_loop
          decfsz    outer,f
          goto      outer_loop
          decfsz    count,f             ;4 chrs sent?
          goto      next_chr            ;no
          incf      FSR,f               ;point to next chr
          goto      next_four           ;yes
abort
          movlw     messag
          movwf     FSR                 ;reset message pointer
          clrf      chr
          clrf      fourchrs
          clrf      flags
          movlw     0x04
          movwf     count
          bcf       RB4                 ;turn off LED
          goto      send_space
record_dot
          movlw     b'00000000'         ;dot = 00
          movwf     chr
          call      record
          addwf     PCL,f               ;w = 1 when last location reached
          goto      dot
          goto      buffer_full         ;signal buffer full
record_dash
          movlw     b'01000000'         ;dash = 01
          movwf     chr
          call      record
          addwf     PCL,f               ;w = 1 when last location reached
          goto      dash
          goto      buffer_full         ;signal buffer full
record_space
          incf      spcecnt,f
          movf      spcecnt,w
          addwf     PCL,f               ;add space counter to PC
          goto      outlow              ;spcecnt = 0, record no more spaces
          goto      outlow              ;        = 1
          goto      rcd3                ;        = 2
          goto      outlow              ;        = 3
          goto      rcd7                ;        = 4
          decf      spcecnt,f           ;spcecnt from 5 to 4
          goto      outlow
rcd3
          movlw     b'10000000'         ;space = 10
          movwf     chr
          call      record
          addwf     PCL,f               ;w = 1 when last location reached
          goto      outlow
          goto      buffer_full         ;signal buffer full
rcd7
          bsf       fourchrs,0          ;change 3 spaces to 7
          btfss     count,2             ;last 4 chrs just saved?
          goto      outlow              ;no so job done
          decf      FSR,f               ;yes so last recorded byte
          movf      fourchrs,w          ;needs to be overwritten
          movwf     INDF
          incf      FSR,f               ;point to next byte again
          goto      outlow

record
          rlf       chr,f
          rlf       fourchrs,f
          rlf       chr,f
          rlf       fourchrs,f
          decfsz    count,f             ;check for 4 chrs
          retlw     0x00                ;less than 4 so carry on
          movlw     messag              ;check for first location
          iorlw     b'11100000'         ;high 3 bits allways read as ones
          subwf     FSR,w             
          btfss     ZERO_BIT
          goto      reset_count         ;not the first location
          btfss     fourchrs,7          ;check for leading spaces
          goto      reset_count         ;no leading spaces
          movlw     0x01                ;yes leading spaces so
          movwf     count               ;return with 1 in count
          retlw     0x00
reset_count
          movlw     0x04
          movwf     count
          movf      fourchrs,w
          movwf     INDF
          incf      FSR,f
          movlw     0xFF                ;last location?
          subwf     FSR,w
          btfss     ZERO_BIT
          retlw     0x00                ;no
          retlw     0x01                ;yes
stoprecord
          movlw     b'11000000'
          movwf     chr
          call      record              ;record end of word
          movlw     b'11000000'
          movwf     chr
again     
          call      record              ;and again for end of message
          btfss     count,2             ;repeat until count is 4
          goto      again
          clrf      spcecnt
          decf      spcecnt,f
          movlw     0xED                ;check to see if the first location
          subwf     FSR,w               ;has just been written
          btfss     ZERO_BIT            
          goto      setzero             ;no
          movf      messag,w            ;check to see if the first location 
          btfsc     ZERO_BIT            ;is clear
          decf      messag,f            ;if so fill with ones
setzero
          movlw     messag              ;reset pointer
          movwf     FSR
          clrf      chr
          clrf      fourchrs
          clrf      flags
          bcf       RB5                 ;turn off LED
          retlw     0x00
buffer_full
          bcf       RB6
          bcf       RB7                 ;output a low
          movlw     0xFF                ;2 end of words
          movwf     spcecnt             ;store in last location
          call      setzero
wait      
          btfss     RA3                 ;wait until record switch is off
          goto      wait
send_space
          btfss     flags,4             ;space just sent?
          goto      clear_message
          bcf       flags,2             ;clear dot last sent flag
          bcf       flags,3             ;clear dash last sent flag
clear_message
          bcf       flags,5             ;clear end of message flag
          bcf       flags,6             ;clear message being sent flag
          btfss     RA2                 ;send message?
          goto      send_message        ;yes
          btfsc     RA3                 ;record message?
          goto      endrecord
          bsf       flags,7             ;recording whilst bit 7 set
          bsf       RB5                 ;used to light LED
          btfsc     flags,4             ;space just sent?
          goto      record_space        ;record a space
          goto      outlow
endrecord
          btfsc     flags,7
          call      stoprecord
outlow
          bcf       RB6
          bcf       RB7                 ;output low on pin 7
          movlw     d'8'
          movwf     outer
outer_space_loop                     
          movlw     d'15' 
          movwf     inner               
          movf      PORTB,w             ;look at port b
          andlw     b'00001111'         ;mask off high 4 bits
          addwf     inner,f             ;add to alter weight
inner_space_loop                        
          decfsz    inner,f            
          goto      inner_space_loop
          movf      PORTA,w             ;look at paddle 
          andlw     b'00000011'         ;mask off bits 2 to 7
          addwf     PCL,f
          goto      both
          goto      send_dash
          goto      send_dot
cont_space
          decfsz    outer,f
          goto      outer_space_loop
          bsf       flags,4             ;space just sent
what_next
          movlw     b'00001111'         ;mask off bits 4 to 7
          andwf     flags,w
          addwf     PCL,f
          goto      send_space          ;0000
          goto      send_dot            ;0001
          goto      send_dash           ;0010
          goto      send_dot            ;0011
          goto      send_space          ;0100
          goto      send_dot            ;0101 should never occur
          goto      send_dash           ;0110
          goto      send_dash           ;0111
          goto      send_space          ;1000
          goto      send_dot            ;1001
          goto      send_dash           ;1010 should never occur
send_dot                                ;1011         
          btfss     flags,2             ;dot last sent?
          bsf       flags,0             ;put a dot in memory
          btfss     flags,4             ;has a space just been sent?
          goto      cont_space          ;no so send rest of space
          btfsc     flags,7             ;recording?
          goto      record_dot          ;store a dot
dot
          bsf       RB6
          bsf       RB7                 ;output a high
          movlw     d'10'
          movwf     outer
outer_dot_loop                          
          movlw     d'24'              
          movwf     inner
inner_dot_loop                          
          decfsz    inner,f           
          goto      inner_dot_loop
          btfss     RA1                 ;look at dash paddle
          bsf       flags,1             ;flag a hit
          decfsz    outer,f
          goto      outer_dot_loop      ;dot sent
          btfsc     flags,6             ;is a message being sent? 
          goto      one_space
          bcf       flags,0             ;clear dot memory
          bsf       flags,2             ;dot last sent
          bcf       flags,3             ;dash not last sent
          bcf       flags,4             ;space not just sent
          clrf      spcecnt             ;set the space counter
          decf      spcecnt,f           ;to FF
          goto      send_space
send_dash         
          btfss     flags,3             ;dash last sent?
          bsf       flags,1             ;put a dash in memory
          btfss     flags,4             ;has a space just been sent?
          goto      cont_space          ;no so send rest of space
          btfsc     flags,7             ;recording?
          goto      record_dash         ;store a dash
dash
          bsf       RB6
          bsf       RB7                 ;output a high
          movlw     d'30'
          movwf     outer
outer_dash_loop                         
          movlw     d'24'            
          movwf     inner
inner_dash_loop                         
          decfsz    inner,f           
          goto      inner_dash_loop
          btfss     RA0                 ;look at dot paddle
          bsf       flags,0             ;flag a hit
          decfsz    outer,f
          goto      outer_dash_loop
          btfsc     flags,6             ;is a message being sent?
          goto      one_space
          bcf       flags,1             ;clear dash memory
          bsf       flags,3             ;dash last sent
          bcf       flags,2             ;dot not last sent
          bcf       flags,4             ;space not just sent
          clrf      spcecnt             ;set the space counter
          decf      spcecnt,f           ;to FF
          goto      send_space
both
          btfsc     flags,4             ;space last sent?
          goto      set_both            ;yes so set both memories
          btfsc     flags,2             ;dot last sent?
          bsf       flags,1             ;yes so store a dash in memory
          btfsc     flags,3             ;dash last sent?
          bsf       flags,0             ;yes so store a dot in memory
          goto      cont_space
set_both
          bsf       flags,0             ;not the space following a dot
          bsf       flags,1             ;or dash so set both memories
          goto      cont_space
;
          org       0x1FF
          goto      init
          END
